/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"

void on_bookmark_edit (GtkMenuItem *menuitem, GaleonBrowser *browser);
void bookmarks_editor_ctree_tree_select_row (GtkCTree *ctree, GList *node, gint column,
					     bookmarks_editor_controls *controls);
void bookmarks_editor_ctree_tree_unselect_row (GtkCTree *ctree, GList *node, gint column,
					       bookmarks_editor_controls *controls);
void bookmarks_editor_multiple_selections_toggled (GtkToggleButton *togglebutton,
						   bookmarks_editor_controls *controls);
void bookmarks_editor_edited (GtkEditable *editable,
			      bookmarks_editor_controls *controls);
void bookmarks_editor_accept (GtkButton *button, bookmarks_editor_controls *controls);
void bookmarks_editor_remove (GtkButton *button, bookmarks_editor_controls *controls);
void bookmarks_editor_move_up (GtkButton *button, bookmarks_editor_controls *controls);
void bookmarks_editor_move_down (GtkButton *button, bookmarks_editor_controls *controls);
void bookmarks_editor_new_item (GtkButton *button, bookmarks_editor_controls *controls);
void bookmarks_editor_new_category (GtkButton *button, 
				    bookmarks_editor_controls *controls);
void bookmarks_editor_new_separator (GtkButton *button, 
				     bookmarks_editor_controls *controls);
gboolean bookmarks_editor_delete_event (GtkWidget *widget, GdkEvent *event,
					bookmarks_editor_controls *controls);
void compact_bookmarks_editor_edit (GtkToggleButton *togglebutton, 
				    bookmarks_editor_controls *controls);
void compact_bookmarks_editor_go (GtkButton *button, bookmarks_editor_controls *controls);
void compact_bookmarks_editor_go_back (GtkButton *button,
				       bookmarks_editor_controls *controls);
void compact_bookmarks_editor_go_forward (GtkButton *button,
					  bookmarks_editor_controls *controls);
void bookmarks_editor_copy (GtkButton *button, bookmarks_editor_controls *controls);
gboolean bookmarks_editor_copy_button_press (GtkWidget *widget, GdkEventButton  *event,
					     bookmarks_editor_controls *controls);
void bookmarks_editor_ctree_drag_data_get (GtkWidget *widget, GdkDragContext *context,
					   GtkSelectionData *selection_data, 
					   guint info, guint time, 
					   bookmarks_editor_controls *controls);
gboolean 
bookmarks_editor_ctree_button_press_event (GtkWidget *widget, GdkEventButton *event, 
					   bookmarks_editor_controls *controls);
void bookmarks_editor_ctree_tree_move (GtkCTree *ctree, GtkCTreeNode *node,
				       GtkCTreeNode *new_parent_node, 
				       GtkCTreeNode *new_sibling_node,
				       bookmarks_editor_controls *controls);
void bookmarks_editor_tree_expand (GtkCTree *ctree, GtkCTreeNode *node, 
				       bookmarks_editor_controls *controls);
void bookmarks_editor_tree_collapse (GtkCTree *ctree, GtkCTreeNode *node, 
				     bookmarks_editor_controls *controls);
void bookmarks_editor_ctree_drag_data_received (GtkWidget *widget,
						GdkDragContext *context,
						gint x, gint y,
						GtkSelectionData *selection_data,
						guint info, guint time, 
						bookmarks_editor_controls *controls);
void on_bookmark_export_netscape (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_bookmark_export_mozilla (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_bookmark_export_other (GtkMenuItem *menuitem, GaleonBrowser *browser);
void bookmark_export_netscape_message (GaleonBrowser *browser, gchar *fname);
void on_bookmark_export_file_ok_button_clicked (GtkButton *button, 
						GaleonBrowser *browser);
void on_bookmark_export_file_cancel_button_clicked (GtkButton *button, 
						    GaleonBrowser *browser);
void bookmark_export_netscape_do (gint reply, gchar *fname);

void
on_bookmark_edit (GtkMenuItem *menuitem, GaleonBrowser *browser) 
{
	browser_edit_bookmarks (browser);
}

/* import given netscape-compatible bookmark filename with pass/fail dialog */
void
bookmark_import_netscape_style(GaleonBrowser * browser, GString * fname)
{
	if (netscape_import_bookmarks(fname))
		gnome_app_message
			(GNOME_APP( ((GaleonBrowser*)window_list->data)->WMain),
			 _("Importing bookmarks failed"));
	else 
		gnome_app_message 
			(GNOME_APP( ((GaleonBrowser*)window_list->data)->WMain), 
			 _("Bookmarks successfully imported/merged"));
}

void
on_bookmark_import_netscape (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	GString *fname = g_string_new (NULL);

	g_string_assign (fname, g_get_home_dir ());
	g_string_append (fname, NETSCAPE_BOOKMARKS);
       	
	bookmark_import_netscape_style(browser, fname);

	g_string_free (fname, TRUE);
}

void
on_bookmark_import_mozilla (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	GString * fname = g_string_new (NULL);

	g_string_assign (fname, g_get_home_dir ());
	g_string_append (fname, MOZILLA_BOOKMARKS);

	bookmark_import_netscape_style(browser, fname);

	g_string_free (fname, TRUE);
}

void
on_bookmark_import_other (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	GtkWidget *fs;
	GladeXML *gxml;

	gxml = glade_xml_new(glade_file(), "bookmarkSelect");
	glade_xml_signal_autoconnect_full (gxml,
			(GladeXMLConnectFunc) glade_signal_connect_func,
			browser);
	fs = glade_xml_get_widget(gxml, "bookmarkSelect");
	gtk_widget_show(fs);
}


void
on_bookmark_file_ok_button_clicked(GtkButton *button, GaleonBrowser *browser)
{
	gchar * bookmarkFile;
	GString * temp;

	GtkWidget * fs = glade_lookup_widget(GTK_WIDGET(button), "bookmarkSelect");

	bookmarkFile = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	temp = g_string_new(bookmarkFile);
	bookmark_import_netscape_style(browser, temp);

	gtk_widget_destroy(fs);
	g_string_free(temp, FALSE);
}

void
on_bookmark_file_cancel_button_clicked(GtkButton *button, GaleonBrowser *browser)
{
	GtkWidget *w = glade_lookup_widget(GTK_WIDGET(button), "bookmarkSelect");
	gtk_widget_destroy (w);
}

void 
on_bookmark_activate (GtkMenuItem *menuitem, gpointer user_data) 
{
        GtkWidget *WMain = glade_lookup_widget (GTK_WIDGET (menuitem), "WMain");
        GaleonBrowser *browser = (GaleonBrowser*) gtk_object_get_data
		(GTK_OBJECT (WMain), "mozilla");

	browser_load_url(browser, user_data);
}

void
on_bookmark_add (GtkMenuItem *menuitem, BookMarkItem *p)
{
	
	GtkWidget *WMain = glade_lookup_widget (GTK_WIDGET (menuitem), "WMain");
	GaleonBrowser *browser= gtk_object_get_data (GTK_OBJECT (WMain), "mozilla");
	BookMarkItem *new;	
	gchar *url = gtk_editable_get_chars (GTK_EDITABLE (browser->entry1), 0, -1);
	gchar *name = g_strdup (gtk_moz_embed_get_title (browser->mozEmbed));
	if (name == NULL || strlen(name) == 0) {
		if (name) g_free (name);
		name = g_strdup (_("Untitled"));
	}
	
	new = bookmarks_new_bookmark (SITE, name, url, NULL, NULL);	
	if (url) g_free (url);
	if (name) g_free (name);
	p->list = g_list_append (p->list, new);
	new->parent = p;
	bookmarks_editor->dirty = TRUE;

	if (gnome_config_get_int(CONF_GENERAL_BOOKMARK_TITLE) == PAGE_TITLE) {
		bookmarks_editor_place_tree_item (bookmarks_editor, new);
		bookmarks_save ();
	} else {
		gnome_request_dialog 
			(FALSE, _("Please enter the name of the bookmark"), new->name,
			 100, GTK_SIGNAL_FUNC (bookmarks_string_request_callback),
			 new, GTK_WINDOW (browser->WMain));
	}
}

void
on_bookmark_export_netscape (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	GString *fname = g_string_new (NULL);

	g_string_assign (fname, g_get_home_dir ());
	g_string_append (fname, NETSCAPE_BOOKMARKS);
       	
	bookmark_export_netscape_message (browser, fname->str);

	g_string_free (fname, FALSE);
}

void
on_bookmark_export_mozilla (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	GString *fname = g_string_new (NULL);

	g_string_assign (fname, g_get_home_dir ());
	g_string_append (fname, MOZILLA_BOOKMARKS);

	bookmark_export_netscape_message (browser, fname->str);

	g_string_free (fname, FALSE);
}

void
on_bookmark_export_other (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	GtkWidget *fs;
	GladeXML *gxml;

	gxml = glade_xml_new(glade_file(), "bookmarks_export_file");
	glade_xml_signal_autoconnect_full (gxml,
			(GladeXMLConnectFunc) glade_signal_connect_func,
			browser);
	fs = glade_xml_get_widget(gxml, "bookmarks_export_file");
	gtk_widget_show(fs);
}

void
on_bookmark_export_file_ok_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
	gchar *bookmarkFile;
	GtkWidget *fs = glade_lookup_widget(GTK_WIDGET(button), "bookmarks_export_file");

	bookmarkFile = gtk_file_selection_get_filename (GTK_FILE_SELECTION(fs));
	if (bookmarkFile) {
		bookmark_export_netscape_message (browser, g_strdup (bookmarkFile));
	}
	gtk_widget_destroy(fs);
}

void
on_bookmark_export_file_cancel_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
	GtkWidget *w = glade_lookup_widget
		(GTK_WIDGET(button), "bookmarks_export_file");
	gtk_widget_destroy (w);
}

void
bookmark_export_netscape_message (GaleonBrowser *browser, gchar *fname)
{
	gchar *question = g_strdup_printf (_("File %s will be overwritten.\n"
					     "If you have any bookmark in that file, "
					     "it will be lost.\n\n"
					     "Do you want to continue?"), fname);
	gnome_question_dialog_modal (question,
				     (GnomeReplyCallback) bookmark_export_netscape_do,
				     fname);
	g_free (question);
}

void 
bookmark_export_netscape_do (gint reply, gchar *fname)
{
	if (reply != GNOME_YES) return;
	if (netscape_export_bookmarks (fname, bookmarks_root))
		gnome_ok_dialog (_("Exporting bookmarks failed"));
	else 
		gnome_ok_dialog (_("Bookmarks successfully exported"));
	g_free (fname);
}


/* Bookmarks editor stuff */

void
bookmarks_editor_ctree_tree_select_row (GtkCTree *ctree, GList *node, gint column,
					bookmarks_editor_controls *controls)
{	
	BookMarkItem *b = controls->selection = 
		gtk_ctree_node_get_row_data (ctree, GTK_CTREE_NODE (node));
	if (!b) return;
	controls->lock_edit_controls = TRUE;

	if (controls->notes_text)
		gtk_editable_delete_text (GTK_EDITABLE (controls->notes_text), 0, -1);
	switch (b->type) {
	case SITE:
		if (controls->name_entry) {
			gtk_entry_set_text (GTK_ENTRY (controls->name_entry), b->name);
			gtk_widget_set_sensitive (GTK_WIDGET (controls->name_entry), 
						  TRUE);
		}
		if (controls->url_entry) {
			gtk_entry_set_text (GTK_ENTRY (controls->url_entry), b->url);
			gtk_widget_set_sensitive (GTK_WIDGET (controls->url_entry),
						  TRUE);
		}
		if (controls->nick_entry) {
			gtk_entry_set_text (GTK_ENTRY (controls->nick_entry), b->nick);
			gtk_widget_set_sensitive (GTK_WIDGET (controls->nick_entry),
						  TRUE);
		}
		if (controls->create_toolbar_toggle) {
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON 
						      (controls->create_toolbar_toggle),
						      FALSE);
			gtk_widget_set_sensitive 
				(GTK_WIDGET (controls->create_toolbar_toggle), FALSE);
		}
		if (controls->notes_text) {
			if (b->notes) gtk_text_insert (GTK_TEXT (controls->notes_text), 
						       NULL, NULL, NULL, b->notes, 
						       strlen (b->notes));
			gtk_widget_set_sensitive (GTK_WIDGET (controls->notes_text), 
						  TRUE);
		}
		break;
	case CATEGORY:
		if (controls->name_entry) {
			gtk_entry_set_text (GTK_ENTRY (controls->name_entry), b->name);
			gtk_widget_set_sensitive (GTK_WIDGET (controls->name_entry),
						  TRUE);
		}
		if (controls->url_entry) {
			gtk_entry_set_text (GTK_ENTRY (controls->url_entry), "");
			gtk_widget_set_sensitive (GTK_WIDGET (controls->url_entry), 
						  FALSE);
		}
		if (controls->nick_entry) {
			gtk_entry_set_text (GTK_ENTRY (controls->nick_entry), "");
			gtk_widget_set_sensitive (GTK_WIDGET (controls->nick_entry), 
						  FALSE);
		}
		if (controls->create_toolbar_toggle) {
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON 
						      (controls->create_toolbar_toggle), 
						      b->create_toolbar);
			gtk_widget_set_sensitive 
				(GTK_WIDGET (controls->create_toolbar_toggle), TRUE);
		}
		if (controls->notes_text) {
			if (b->notes) gtk_text_insert (GTK_TEXT (controls->notes_text), 
						       NULL, NULL, NULL, b->notes, 
						       strlen (b->notes));
			gtk_widget_set_sensitive (GTK_WIDGET (controls->notes_text),
						  TRUE);
		}
		break;
	default:
		if (controls->name_entry) {
			gtk_entry_set_text (GTK_ENTRY (controls->name_entry), "");	
			gtk_widget_set_sensitive (GTK_WIDGET (controls->name_entry), 
						  FALSE);
		}
		if (controls->url_entry) {
			gtk_entry_set_text (GTK_ENTRY (controls->url_entry), "");
			gtk_widget_set_sensitive (GTK_WIDGET (controls->url_entry), 
						  FALSE);
		}
		if (controls->nick_entry) {
			gtk_entry_set_text (GTK_ENTRY (controls->nick_entry), "");
			gtk_widget_set_sensitive (GTK_WIDGET (controls->nick_entry), 
						  FALSE);
		}
		if (controls->notes_text) {
			gtk_widget_set_sensitive (GTK_WIDGET (controls->notes_text), 
						  FALSE);
		}
		if (controls->create_toolbar_toggle) {
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON 
						      (controls->create_toolbar_toggle),
						      FALSE);
			gtk_widget_set_sensitive 
				(GTK_WIDGET (controls->create_toolbar_toggle), FALSE);
		}
		break;
	}

	if (gtk_ctree_node_is_visible (ctree, b->tree_item) != GTK_VISIBILITY_FULL) 
		gtk_ctree_node_moveto (ctree, b->tree_item, 0, -1, -1);
	
	controls->lock_edit_controls = FALSE;
}

void
bookmarks_editor_ctree_tree_unselect_row (GtkCTree *ctree, GList *node, gint column,
					  bookmarks_editor_controls *controls)
{	
	controls->lock_edit_controls = TRUE;
	controls->selection = NULL;

	if (controls->name_entry) {
		gtk_entry_set_text (GTK_ENTRY (controls->name_entry), "");	
		gtk_widget_set_sensitive (GTK_WIDGET (controls->name_entry), 
					  FALSE);
	}
	if (controls->url_entry) {
		gtk_entry_set_text (GTK_ENTRY (controls->url_entry), "");
		gtk_widget_set_sensitive (GTK_WIDGET (controls->url_entry), 
					  FALSE);
	}
	if (controls->nick_entry) {
		gtk_entry_set_text (GTK_ENTRY (controls->nick_entry), "");
		gtk_widget_set_sensitive (GTK_WIDGET (controls->nick_entry), 
					  FALSE);
	}
	if (controls->notes_text) {
		gtk_editable_delete_text (GTK_EDITABLE (controls->notes_text), 0, -1);
		gtk_widget_set_sensitive (GTK_WIDGET (controls->notes_text), 
					  FALSE);
	}
	if (controls->create_toolbar_toggle) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON 
					      (controls->create_toolbar_toggle), FALSE);
		gtk_widget_set_sensitive 
			(GTK_WIDGET (controls->create_toolbar_toggle), FALSE);
	}
	controls->lock_edit_controls = FALSE;
}

void
bookmarks_editor_multiple_selections_toggled (GtkToggleButton *togglebutton,
					      bookmarks_editor_controls *controls)
{
	if (gtk_toggle_button_get_active (togglebutton))
		gtk_clist_set_selection_mode
			(GTK_CLIST (controls->ctree), GTK_SELECTION_MULTIPLE);
	else
		gtk_clist_set_selection_mode
			(GTK_CLIST (controls->ctree), GTK_SELECTION_SINGLE);
}

void
bookmarks_editor_edited (GtkEditable *editable, bookmarks_editor_controls *controls)
{
	if (!controls->lock_edit_controls && controls->selection) {
		BookMarkItem *b = controls->selection;
		gchar *name = NULL;
		gchar *url = NULL;
		gchar *nick = NULL;
		gboolean create_toolbar = FALSE;
		gchar *notes = NULL;
		if (controls->name_entry)
			name = gtk_editable_get_chars 
				(GTK_EDITABLE (controls->name_entry), 0, -1);
		if (controls->url_entry)
			url = gtk_editable_get_chars 
				(GTK_EDITABLE (controls->url_entry), 0, -1);
		if (controls->nick_entry)
			nick = gtk_editable_get_chars 
				(GTK_EDITABLE (controls->nick_entry), 0, -1);
		if (controls->create_toolbar_toggle)
			create_toolbar = gtk_toggle_button_get_active 
				(GTK_TOGGLE_BUTTON (controls->create_toolbar_toggle));
		if (controls->notes_text)
			notes = gtk_editable_get_chars 
				(GTK_EDITABLE (controls->notes_text), 0, -1);
		switch (b->type) {
		case SITE:
			if (controls->name_entry) {
				if (b->name) g_free (b->name);
				b->name = name;
			}
			if (controls->url_entry) {
				if (b->url) g_free (b->url);
				b->url = url;
			}
			if (controls->nick_entry) {
				if (b->nick) g_free (b->nick);
				b->nick = nick;
			}
			if (controls->notes_text) {
				if (b->notes) g_free (b->notes);
				b->notes = notes;
			}
			break;
		case CATEGORY:
			if (controls->name_entry) {
				if (b->name) g_free (b->name);
				b->name = name;
			}
			if (url) g_free (url);
			if (nick) g_free (nick);
			if (controls->create_toolbar_toggle) {
				b->create_toolbar = create_toolbar;
			}
			if (controls->notes_text) {
				if (b->notes) g_free (b->notes);
				b->notes = notes;
			}
			break;
		default:
			break;
		}

		bookmarks_editor_update_tree_item (controls, b);
		controls->dirty = TRUE;
	}
}

void 
bookmarks_editor_accept (GtkButton *button, bookmarks_editor_controls *controls)
{
	gtk_widget_hide (controls->dialog);
	if (controls->dirty) {
		if (controls->save_function)
			controls->save_function();
	}
}

gboolean
bookmarks_editor_delete_event (GtkWidget *widget, GdkEvent *event,
			       bookmarks_editor_controls *controls)
{
	bookmarks_editor_accept (NULL, controls);
	return TRUE;
}

void 
bookmarks_editor_remove (GtkButton *button, bookmarks_editor_controls *controls)
{
	GList *selection_list = NULL;
	GList *l;
	BookMarkItem *next_to_select = NULL;
	for (l = GTK_CLIST(controls->ctree)->selection; l != NULL; l = g_list_next (l)) {
		selection_list = g_list_prepend 
			(selection_list,  gtk_ctree_node_get_row_data 
			 (GTK_CTREE (controls->ctree), l->data));
	}
	if ((controls->selection != NULL) && g_list_next (selection_list) == NULL
	    && (controls->selection->parent != NULL)) { 
		/* there was only one bookmark selected */
		GList *next_to_select_pos = g_list_next (
			g_list_find 
			(controls->selection->parent->list, controls->selection));
                        if (next_to_select_pos) 
                                next_to_select = next_to_select_pos->data;              
	}
	gtk_clist_freeze (GTK_CLIST (controls->ctree));
	for (l = selection_list; l != NULL; l = g_list_next (l)) {
		BookMarkItem *b = l->data;
		gtk_ctree_remove_node (GTK_CTREE (controls->ctree), b->tree_item);
		bookmarks_remove_recursively (b);
		controls->dirty = TRUE;
        }
	gtk_clist_thaw (GTK_CLIST (controls->ctree));
		
	if (next_to_select)
		gtk_ctree_select (GTK_CTREE (controls->ctree), 
				  next_to_select->tree_item);
	g_list_free (selection_list);
}

void 
bookmarks_editor_move_up (GtkButton *button, bookmarks_editor_controls *controls) 
{
	if (controls->selection) {
		bookmarks_move_bookmark (controls, controls->selection, 0);
		controls->dirty = TRUE;
	}
	    
}

void 
bookmarks_editor_move_down (GtkButton *button, bookmarks_editor_controls *controls) 
{
	if (controls->selection) {
		bookmarks_move_bookmark (controls, controls->selection, 1);
		controls->dirty = TRUE;
	}
	    
}

void
compact_bookmarks_editor_edit (GtkToggleButton *togglebutton, 
			       bookmarks_editor_controls *controls) 
{
	if (gtk_toggle_button_get_active (togglebutton))
		gtk_widget_show (controls->edit_frame);
	else
		gtk_widget_hide (controls->edit_frame);
}

void 
bookmarks_editor_new_item (GtkButton *button, bookmarks_editor_controls *controls) 
{
	BookMarkItem *new = bookmarks_new_bookmark 
		(SITE, _("New bookmark item"), _("where://do.you.want.to.go"), NULL, NULL);
	BookMarkItem *near = controls->selection;
	if (!near) near = controls->root_bookmark;
	bookmarks_insert_bookmark (new, near);
	bookmarks_editor_place_tree_item (controls, new);
	controls->dirty = TRUE;
	gtk_ctree_select (GTK_CTREE (controls->ctree), new->tree_item);
}

void 
bookmarks_editor_new_category (GtkButton *button, bookmarks_editor_controls *controls) 
{
	BookMarkItem *new = bookmarks_new_bookmark 
		(CATEGORY, _("New bookmark category"), NULL, NULL, NULL);
	BookMarkItem *near = controls->selection;
	if (!near) near = controls->root_bookmark;
	bookmarks_insert_bookmark (new, near);
	bookmarks_editor_place_tree_item (controls, new);
	controls->dirty = TRUE;
	gtk_ctree_select (GTK_CTREE (controls->ctree), new->tree_item);
}

void 
bookmarks_editor_new_separator (GtkButton *button, bookmarks_editor_controls *controls) 
{
	BookMarkItem *new = bookmarks_new_bookmark 
		(SEPARATOR, _("Separator"), NULL, NULL, NULL);
	BookMarkItem *near = controls->selection;
	if (!near) near = controls->root_bookmark;
	bookmarks_insert_bookmark (new, near);
	bookmarks_editor_place_tree_item (controls, new);
	controls->dirty = TRUE;
	gtk_ctree_select (GTK_CTREE (controls->ctree), new->tree_item);
}

void
compact_bookmarks_editor_go (GtkButton *button, bookmarks_editor_controls *controls)
{
	GSList *pos;

	if (!controls->selection || (controls->selection->type != SITE))
		return;

	/* Check if the browser that is selected is already open */
	pos = g_slist_find (window_list, controls->browser);
	if (!pos) controls->browser = NULL;

	/* create a new browser if needed */
	if (!controls->browser && controls->selection) {
		controls->browser = browser_create ();
	}
	
	if (controls->selection) {
		browser_load_url (controls->browser, controls->selection->url);
	}
}

void
compact_bookmarks_editor_go_forward (GtkButton *button,
				     bookmarks_editor_controls *controls)
{
	if (controls->selection && controls->selection->parent) {
		GList *dest_pos = g_list_next (
			g_list_find (controls->selection->parent->list, 
				     controls->selection));
		if (dest_pos) {
			BookMarkItem *dest = dest_pos->data;
			if (dest) {
				gtk_ctree_select (GTK_CTREE (controls->ctree),
						  dest->tree_item);
				compact_bookmarks_editor_go (button, controls);
			}
		}
	}
}

void
compact_bookmarks_editor_go_back (GtkButton *button, bookmarks_editor_controls *controls)
{
	if (controls->selection && controls->selection->parent) {
		GList *dest_pos = g_list_previous (
			g_list_find (controls->selection->parent->list, 
				     controls->selection));
		if (dest_pos) {
			BookMarkItem *dest = dest_pos->data;
			if (dest) {
				gtk_ctree_select (GTK_CTREE (controls->ctree),
						  dest->tree_item);
				compact_bookmarks_editor_go (button, controls);
			}
		}
	}
}

gboolean
bookmarks_editor_copy_button_press (GtkWidget *widget, GdkEventButton *event,
				    bookmarks_editor_controls *controls)
{
	if (event->button == 3) {
		bookmarks_editor_copy (GTK_BUTTON (widget), controls);
	}
	return FALSE;
}

void
bookmarks_editor_copy (GtkButton *button, bookmarks_editor_controls *controls)
{
	if (controls->selection) {
		GtkMenu *destination_menu = 
			bookmarks_create_copy_menu (bookmarks_editor, 
						    controls->selection);
		gnome_popup_menu_do_popup_modal
			(GTK_WIDGET (destination_menu), menu_position_under_widget, 
			 GTK_WIDGET (button), NULL, NULL);
		gtk_widget_destroy (GTK_WIDGET (destination_menu));
		bookmarks_save ();
	}
}

void
bookmarks_editor_ctree_drag_data_get (GtkWidget *widget, GdkDragContext *context,
				      GtkSelectionData *selection_data, 
				      guint info, guint time, 
				      bookmarks_editor_controls *controls)
{
	if (controls->last_pressed) {
		gchar *mem;
		switch (info) {
		case DND_TARGET_GALEON_BOOKMARK:
			if (controls->last_pressed) {
				mem = bookmarks_item_to_string (controls->last_pressed);
				gtk_selection_data_set 
					(selection_data, selection_data->target,
					 8, mem, strlen (mem));
			}
			break;
		case DND_TARGET_STRING:
			if (controls->last_pressed->type == CATEGORY) {
				if (controls->last_pressed->name
				    && (strlen (controls->last_pressed->name) > 0)) {
					gtk_selection_data_set 
						(selection_data, selection_data->target,
						 8, controls->last_pressed->name, 
						 strlen (controls->last_pressed->name));
				}
				break;
			}
		case DND_TARGET_NETSCAPE_URL:
		case DND_TARGET_GALEON_URL:
			if (controls->last_pressed->url 
			    && (strlen (controls->last_pressed->url) > 0)) {
				gtk_selection_data_set 
					(selection_data, selection_data->target,
					 8, controls->last_pressed->url, 
					 strlen (controls->last_pressed->url));
			}
			break;
		case DND_TARGET_TEXT_URI_LIST:
			if (controls->last_pressed->type == CATEGORY) {
				GList *l;
				gchar *text = g_strdup ("");
				for (l = controls->last_pressed->list; 
				     l != NULL; 
				     l = g_list_next(l)) {
					BookMarkItem *item = l->data;
					if (item->url && (strlen (item->url) > 0)) {
						gchar *prevtext = text;
						text = g_strconcat (text, item->url, 
								    "\r\n", NULL);
						g_free (prevtext);
					}
				}
				gtk_selection_data_set 
					(selection_data, selection_data->target,
					 8, text, strlen (text));
				g_free (text);
			} else if (controls->last_pressed->url 
				   && (strlen (controls->last_pressed->url) > 0)) {
				gchar *text = g_strconcat 
					(controls->last_pressed->url, "\r\n", NULL);
				gtk_selection_data_set 
					(selection_data, selection_data->target,
					 8, text, strlen (text));
				g_free (text);
			}
			break;
		default:
			g_warning ("Unknown DND type");
			break;
		}
	} 
}

void 
bookmarks_editor_ctree_tree_move (GtkCTree *ctree, GtkCTreeNode *node,
				  GtkCTreeNode *new_parent_node, 
				  GtkCTreeNode *new_sibling_node, 
				  bookmarks_editor_controls *controls)
{
	BookMarkItem *source = gtk_ctree_node_get_row_data (ctree, node);
	BookMarkItem *new_parent = gtk_ctree_node_get_row_data (ctree, new_parent_node);
	BookMarkItem *new_sib = gtk_ctree_node_get_row_data (ctree, new_sibling_node);
	GList *new_sib_list_node;
	gint new_pos = 0;

	g_return_if_fail (source && source->parent);
	
	if (! new_parent) new_parent = controls->root_bookmark;
	
	new_sib_list_node = g_list_find (new_parent->list, new_sib);
	if (new_sib_list_node) {
		new_pos = g_list_position (new_parent->list, new_sib_list_node);
	}

	source->parent->list = g_list_remove (source->parent->list, source);
	source->parent = new_parent;
	new_parent->list = g_list_insert (new_parent->list, source, new_pos);
	/* DEBUG only */
#if 0
	g_assert (g_list_find (source->parent->list, source));
	g_assert (g_list_position 
		  (source->parent->list, g_list_find (source->parent->list, source)) 
		  == new_pos);
#endif
	controls->dirty = TRUE;
}

void
on_bookmark_copy (GtkMenuItem *menuitem, BookMarkItem *dest)
{
	BookMarkItem *item = gtk_object_get_user_data (GTK_OBJECT (menuitem));
	BookMarkItem *new;
	g_assert (item != NULL);
	g_assert (dest->type == CATEGORY);
	new = bookmarks_copy_bookmark (item);
	new->parent = dest;
	dest->list = g_list_append (dest->list, new);
	bookmarks_editor->dirty = TRUE;
	bookmarks_editor_place_tree_item (bookmarks_editor, new);
}

gboolean 
bookmarks_editor_ctree_button_press_event (GtkWidget *widget, GdkEventButton  *event, 
					   bookmarks_editor_controls *controls)
{
	gint row, column;
	gtk_clist_get_selection_info (GTK_CLIST (widget), event->x, event->y,
				      &row, &column);
	controls->last_pressed = gtk_ctree_node_get_row_data 
		(GTK_CTREE (widget), gtk_ctree_node_nth (GTK_CTREE (widget), row));
	return FALSE;
}

void 
bookmarks_editor_tree_expand (GtkCTree *ctree, GtkCTreeNode *node, 
			      bookmarks_editor_controls *controls)
{
	if (controls->last_pressed) {
		controls->last_pressed->expanded = TRUE;
		controls->dirty = TRUE;
	}
}

void 
bookmarks_editor_tree_collapse (GtkCTree *ctree, GtkCTreeNode *node, 
				     bookmarks_editor_controls *controls)
{
	if (controls->last_pressed) {
		controls->last_pressed->expanded = FALSE;
		controls->dirty = TRUE;
	}
}

void
bookmarks_editor_ctree_drag_data_received (GtkWidget *widget, GdkDragContext *context,
					   gint x, gint y,
					   GtkSelectionData *selection_data,
					   guint info, guint time, 
					   bookmarks_editor_controls *controls)
{
	BookMarkItem *b = NULL;
	gchar *s, *t;
	switch(info)
	{
	case DND_TARGET_GALEON_BOOKMARK:
		b = bookmarks_item_from_string (selection_data->data);
		break;
	case DND_TARGET_TEXT_URI_LIST:
		b = bookmarks_new_bookmark 
			(CATEGORY, "New category", NULL, NULL, NULL);
		s = selection_data->data;
		while ((t = strstr (s, "\r\n"))) {
			gchar *url = g_strndup (s, t-s);
			BookMarkItem *b2 = bookmarks_new_bookmark 
				(SITE, NULL, url, NULL, NULL);
			b2->parent = b;
			b->list = g_list_append (b->list, b2);
			g_free (url);
			s = t+1;
		};
		break;
	case DND_TARGET_NETSCAPE_URL:
	case DND_TARGET_GALEON_URL:
	case DND_TARGET_STRING:
		b = bookmarks_new_bookmark 
			(SITE, NULL, selection_data->data, NULL, NULL);
		break;
	default:
		g_warning ("Unknown DND type");
		break;
	}
	if (b) {
		BookMarkItem *near = NULL;
		gint row, column;
		gtk_clist_get_selection_info (GTK_CLIST (controls->ctree), x, y,
					      &row, &column);
		near = gtk_ctree_node_get_row_data 
			(GTK_CTREE (controls->ctree), gtk_ctree_node_nth
			 (GTK_CTREE (widget), row));
		if (!near) near = controls->root_bookmark;
		bookmarks_insert_bookmark (b, near);
		bookmarks_editor_place_tree_item (controls, b);
		controls->dirty = TRUE;
		gtk_ctree_select (GTK_CTREE (controls->ctree), b->tree_item);		
	}
}

/* End of bookmarks editor stuff */

