/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"

/* local function prototyprs */
gint timeout_callback(GaleonBrowser *browser);
void on_new_window1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_open1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_open_url1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_save_as1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_exit1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_print1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_close1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
gboolean on_Bbf_button_press_event (GtkWidget *widget, GdkEventButton  *event,
				    GaleonBrowser *browser);
void on_BBack_leave (GtkButton *button, GaleonBrowser *browser);
void on_BForward_leave (GtkButton *button, GaleonBrowser *browser);
void on_BStart_clicked (GtkButton *button, GaleonBrowser *browser);
void on_BBack_clicked (GtkButton *button, GaleonBrowser *browser);
void on_BForward_clicked (GtkButton *button, GaleonBrowser *browser);
void on_BBack_pressed (GtkButton *button, GaleonBrowser *browser);
void on_BForward_pressed (GtkButton *button, GaleonBrowser *browser);
void on_BRefresh_clicked (GtkButton *button, GaleonBrowser *browser);
void on_BStop_clicked (GtkButton *button, GaleonBrowser *browser);
gboolean on_entry1_key_press_event (GtkWidget *widget, GdkEventKey *event,
				    GaleonBrowser *browser);
void on_bGo_clicked (GtkButton *button, GaleonBrowser *browser);
void on_back1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_forward1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_stop1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_reload1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_start_page_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_zoom_spin_changed(GtkEditable *editable, GaleonBrowser *browser);
gint zoom_timeout_callback (GaleonBrowser *browser);
void on_view_source_mode_activate (GtkCheckMenuItem *menuitem,
				   GaleonBrowser *browser);
void on_show_temporary_bookmarks_activate (GtkMenuItem *menuitem,
					   GaleonBrowser *browser);
void on_add_temporary_bookmark_activate (GtkMenuItem *menuitem,
					 GaleonBrowser *browser);
void on_view_toolbar_activate (GtkCheckMenuItem *menuitem, GaleonBrowser *browser);
void on_view_statusbar_activate (GtkCheckMenuItem *menuitem, GaleonBrowser *browser);
void on_view_menubar_activate (GtkCheckMenuItem *menuitem, GaleonBrowser *browser);
void on_view_fullscreen_activate (GtkCheckMenuItem *menuitem, GaleonBrowser *browser);
void on_location_drag_data_get (GtkWidget *widget, GdkDragContext *context,
				GtkSelectionData *selection_data, 
				guint info, guint time, 
				GaleonBrowser *browser);
void on_bookmarks_menuitem_drag_data_received (GtkWidget *widget, 
					       GdkDragContext *drag_context, gint x,
					       gint y, GtkSelectionData *selection_data, 
					       guint info, guint time, 
					       GaleonBrowser *browser);
void on_url_entry_drag_data_received (GtkWidget *widget, 
				      GdkDragContext *drag_context, gint x, gint y,
				      GtkSelectionData *selection_data, guint info,
				      guint time, GaleonBrowser *browser);
gboolean on_zoom_spin_key_press_event (GtkWidget *widget, GdkEventKey *event,
				       GaleonBrowser *browser);

GTimer *zoom_timer = NULL;
#define ZOOM_DELAY 0.75 

/** 
 * timeout_callback: Gtk+ interval function used for timer handling
 * This timeout handler is currently only enabled when the Back/Forward
 * buttons are held down 
 */
gint timeout_callback(GaleonBrowser *browser)
{
#ifdef B_HISTORY
        if (BTimer != NULL) /* holding down the "Back" button? */
	{
                if (g_timer_elapsed(BTimer, NULL) >= B_HISTORY_SECS) {
			GtkMenu *menu;
			g_timer_stop(BTimer);
			g_timer_destroy(BTimer);
			BTimer = NULL;
			timertag = -1;

			menu = create_back_menu (browser);

			if (menu)
			{
				gnome_popup_menu_do_popup_modal
					(GTK_WIDGET(menu), menu_position_under_widget, 
					 browser->BBack,
					 NULL, NULL);
				gtk_widget_destroy(GTK_WIDGET(menu));
			}
			
			return TRUE;

                }
        }

        if (FTimer != NULL) /* holding down the "Forward" button? */
	{
	        if (g_timer_elapsed(FTimer, NULL) >= B_HISTORY_SECS) 
		{
			GtkMenu *menu;
			g_timer_stop(FTimer);
			g_timer_destroy(FTimer);
			FTimer = NULL;
			timertag = -1;

			menu = create_forward_menu (browser);

			if (menu)
			{
				gnome_popup_menu_do_popup_modal
					(GTK_WIDGET(menu), menu_position_under_widget, 
					 browser->BForward,
					 NULL, NULL);
				gtk_widget_destroy(GTK_WIDGET(menu));
			}

			return TRUE;
                }
	}
#endif
        return TRUE;  /* return FALSE to also disable this callback */
}


/** 
 * on_new_window_activate: open a new window
*/
void
on_new_window1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	browser_create_default (FALSE);
}


/** 
 * on_new_window_activate: show open file dialog
*/
void
on_open1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	browser_show_open_dialog(browser);
}

/** 
 * on_new_window_activate: show open url dialog
*/
void
on_open_url1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	browser_show_openurl_dialog(browser);
}

/** 
 * on_new_window_activate: open url dialog ok button pressed
*/
void
on_open_url_ok_cb    (gchar *string, GaleonBrowser *browser)
{
	if (string) {
		gchar *string2 = bookmarks_parse_nick (string, NULL);
		if (string2) {
			browser_load_url (browser, string2);
			g_free(string2);
		} else {
			browser_load_url (browser, string);
		}
	}
}

/** 
 * on_save_as1_actuvate: show save dialog
 */
void
on_save_as1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	browser_show_save_dialog (browser, NULL);
}

/** 
 * on_exit1_activate: exit galeon
 */
void
on_exit1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	galeon_exit();
}

/** 
 * on_print1_activate: print document
 */
void
on_print1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	mozilla_print(browser->mozEmbed);
	gnome_error_dialog("Not yet implemented.");
}


/** 
 * on_print1_activate: close current window
 */
void
on_close1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser) 
{
	browser_close(browser);
}


/** 
 * on_Bbf_button_press_event:
 */
gboolean
on_Bbf_button_press_event (GtkWidget *widget, GdkEventButton  *event, GaleonBrowser *browser)
{
	if (event->button == 3) {
		GtkMenu *menu = NULL;
		if (widget == browser->BForward) menu = create_forward_menu (browser);
		if (widget == browser->BBack) menu = create_back_menu (browser);
		g_assert (menu != NULL);
		gnome_popup_menu_do_popup_modal
			(GTK_WIDGET (menu), menu_position_under_widget, 
			 widget, NULL, NULL);
		gtk_widget_destroy (GTK_WIDGET (menu));
		return FALSE;
	}	
	return TRUE;
}

/** 
 * on_BBack_leave: back button left
 */
void
on_BBack_leave (GtkButton *button, GaleonBrowser *browser)
{
#ifdef B_HISTORY
	if (BTimer != NULL)
	{
	       g_timer_stop(BTimer);
	       g_timer_destroy(BTimer);
	       BTimer = NULL;
	}

        if (timertag != -1)
	{
	        gtk_timeout_remove(timertag);
	        timertag = -1;
  	}
#endif
}

/** 
 * on_BForward_leave: forward button left
*/
void
on_BForward_leave (GtkButton *button, GaleonBrowser *browser)
{
#ifdef B_HISTORY
       if (FTimer != NULL)
       {
               g_timer_stop(FTimer);
               g_timer_destroy(FTimer);
               FTimer = NULL;
       }

       if (timertag != -1)
       {
               gtk_timeout_remove(timertag);
               timertag = -1;
       }
#endif
}


/** 
 * on_BStart_leave: start button clicked
*/
void
on_BStart_clicked (GtkButton *button, GaleonBrowser *browser)
{
  gchar *startpage = gnome_config_get_string("/galeon/General/startpage=www.gnome.org");

  if (startpage != NULL && strlen(startpage) != 0) {
	  browser_load_url(browser, startpage);
  } else {
    gnome_error_dialog(_("You must specify a Start Page in the Preferences dialog!"));
  }

  g_free(startpage);
}


/*
 * How the Back/Forward history works - Nate Case <nd@kracked.com>
 *
 * This is implemented using some standard gtk+ callbacks and timers
 * with a pop-up menu.
 * 
 * The Gtk+ event "pressed" is activated as soon as the mouse clicks it,
 * without caring if they release the button somewhere else like "clicked".
 *
 * The Gtk+ event "leave" is activated when the mouse cursor leaves the
 * button area.
 * 
 * When "pressed" is activated, we immediately start a timer.  This timer
 * is reset and disabled when either "clicked" or "leave" signals.  If
 * the timer reaches X ms, then they have been holding the button down
 * for that amount of time and we pop-up a menu containing the history.
*/

/** 
 * on_BBack_clicked: back button clicked
*/
void
on_BBack_clicked (GtkButton *button, GaleonBrowser *browser)
{
#ifdef B_HISTORY
	if (BTimer != NULL)
	{
		g_timer_stop(BTimer);
		g_timer_destroy(BTimer);
		BTimer = NULL;
	}

	if (timertag != -1)
	{
		gtk_timeout_remove(timertag);
		timertag = -1;
	}
#endif
	gtk_moz_embed_go_back(browser->mozEmbed);
}


/** 
 * on_BForward_clicked: forward button clicked
*/
void
on_BForward_clicked (GtkButton *button, GaleonBrowser *browser)
{
#ifdef B_HISTORY
	if (FTimer != NULL)
	{
		g_timer_stop(FTimer);
		g_timer_destroy(FTimer);
		FTimer = NULL;
	}
	
	if (timertag != -1)
	{
		gtk_timeout_remove(timertag);
		timertag = -1;
	}
#endif

	gtk_moz_embed_go_forward(browser->mozEmbed);
}

/** 
 * on_BBack_pressed: back button pressed
*/
void
on_BBack_pressed (GtkButton *button, GaleonBrowser *browser)
{
#ifdef B_HISTORY
	if (BTimer == NULL)
	{
	        BTimer = g_timer_new();
	        g_timer_start(BTimer);
	        if (timertag == -1)
	                timertag = gtk_timeout_add(100, timeout_callback, (gpointer) browser);
	}
#endif
}

/** 
 * on_BForward_pressed: forward button pressed
*/
void
on_BForward_pressed (GtkButton *button, GaleonBrowser *browser)
{
#ifdef B_HISTORY
	if (FTimer == NULL)
	{
	        FTimer = g_timer_new();
	        g_timer_start(FTimer);
 	        if (timertag == -1)
	        	timertag = gtk_timeout_add(100, timeout_callback, (gpointer) browser);
	}
#endif
}

/** 
 * on_BRefresh_clicked: refresh button clicked
*/
void
on_BRefresh_clicked (GtkButton *button, GaleonBrowser *browser)
{
  gtk_moz_embed_reload(browser->mozEmbed, GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
}


/** 
 * on_BStop_clicked: stop button clicked
 */
void
on_BStop_clicked (GtkButton *button, GaleonBrowser *browser)
{
  gtk_moz_embed_stop_load(browser->mozEmbed);
}

/** 
 * on_entry1_key_press_event: key pressed in the url entry
 */
gboolean
on_entry1_key_press_event (GtkWidget *widget, GdkEventKey *event, GaleonBrowser *browser)
{
	/* FIXME: cleanup ? */
	int completion_type = gnome_config_get_int("/galeon/General/completion_type=0");

	if ((event->keyval == GDK_Return) || (event->keyval == GDK_KP_Enter)) {
		on_bGo_clicked (NULL, browser);
		event->keyval = GDK_VoidSymbol;
	}

	if (completion_type == 0) {
		if (event->string[0] > 32 && event->string[0] < 126) {
			gchar *text = gtk_editable_get_chars (browser->entry1, 0, -1);
			gchar *new_text;
			gchar *common_prefix;
			
			/*
			 * There will be a preference for this when it is more useful
			 * I want it to be easier to disable, perhaps an ignore after
			 * backspace?  And it should be able to autodetect ftp, etc
			 * from the history list.  Right now, if you want it, you can
			 * set it to 0 in your ~/.gnome/galeon file.
			 */
			if (!gnome_config_get_int("/galeon/General/autohttp=1") &&
			   text[0] != 'h') {
				new_text = g_strconcat("http://", text, NULL);
				g_free(text);
				text = new_text;
			}
			common_prefix = auto_completion_complete_url(text);
			if (common_prefix) {
				int text_len = strlen(text);
				int i;
				
				g_free(common_prefix);
				gtk_entry_set_text(GTK_ENTRY(browser->entry1),
						   (gchar *) posible_completions->data);
				/*
				 * Do not complete past directories or domains.
				 * This makes it more *gasp* ie like.
				 * I will add a pref for this later too.
				 */
				g_free(text);
				text = gtk_editable_get_chars(browser->entry1, 0, -1);
				for(i = text_len; i < strlen(text); i++) {
					if(text[i] == '/') {
						if(i + 1 < strlen(text))
							text[++i] = '\0';
						gtk_entry_set_text(GTK_ENTRY(browser->entry1), text);
						break;
					}
				}
				gtk_editable_select_region(browser->entry1, text_len, -1);
				gtk_editable_set_position(browser->entry1, text_len - 1);
			}
			g_free(text);
		}
	} else if (completion_type == 1) {
		static gchar *before_completion = NULL;
		if (event->keyval == GDK_Tab) {
			gchar *text = gtk_editable_get_chars(browser->entry1, 0, -1);
			gchar *common = auto_completion_complete_url_extended(text);
			if (common) {
				if (!before_completion) 
					before_completion = g_strdup(text);
				gtk_entry_set_text(GTK_ENTRY(browser->entry1), common);
				g_free(common);
				auto_completion_display_alternatives
					(GTK_WIDGET (browser->entry1));
			}
			event->keyval = GDK_VoidSymbol;
		} else if (event->keyval == GDK_Escape) {
		        auto_completion_reset();
                        if (before_completion) {
			    gtk_entry_set_text(GTK_ENTRY(browser->entry1), before_completion);
			    g_free(before_completion);
			    before_completion = NULL;
		    }
	    } else {
		    auto_completion_reset();
		    if (before_completion) {
			    g_free(before_completion);
			    before_completion = NULL;
		    }
	    }
	}

	if ( (event->keyval == GDK_Left) || (event->keyval == GDK_Right) )
		event->keyval = GDK_VoidSymbol;

	return FALSE;
}

/** 
 * on_bGo_clicked: go button clicked
*/
void
on_bGo_clicked (GtkButton *button, GaleonBrowser *browser)
{
	gchar *text = gtk_editable_get_chars (browser->entry1, 0, -1);
	gchar *text2 = bookmarks_parse_nick (text, NULL);
	if (text2) {
		g_free (text);
		text = text2;
	}
	browser_load_url (browser, text);
	g_free (text);
}

/** 
 * on_back1_activate: back menu activated
*/
void
on_back1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
  gtk_moz_embed_go_back(browser->mozEmbed);
}


/** 
 * on_forward1_activate: forward menu activated
*/
void
on_forward1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
  gtk_moz_embed_go_forward(browser->mozEmbed);
}


/** 
 * on_stop1_activate: stop menu activated
*/
void
on_stop1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
  gtk_moz_embed_stop_load(browser->mozEmbed);
}


/** 
 * on_reload1_activate: reload menu activated
*/
void
on_reload1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
  gtk_moz_embed_reload(browser->mozEmbed, GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
}

/** 
 * on_start_page_activate: start page menu activated
 */
void
on_start_page_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	on_BStart_clicked(GTK_BUTTON(browser->BStart), browser);
}

/**
 * Changes the zoom if enough time time has passed 
 */
gint 
zoom_timeout_callback (GaleonBrowser *browser)
{
	if (zoom_timer == NULL) return FALSE;
	if (g_timer_elapsed (zoom_timer, NULL) >= ZOOM_DELAY) {
		gint value = atoi (gtk_editable_get_chars (browser->zoom_spin, 0, -1));
		g_timer_destroy (zoom_timer);
		zoom_timer = NULL;
		mozilla_set_zoom (browser->mozEmbed, (float) value / 100);
		return FALSE;
	}
	return TRUE;
}

/** 
 * on_zoom_spin_changed: zoom spin value changed. Starts the zoom timer
 */
void 
on_zoom_spin_changed (GtkEditable *editable, GaleonBrowser *browser)
{
	if (zoom_timer != NULL) g_timer_destroy (zoom_timer);
	zoom_timer = g_timer_new();
	g_timer_start (zoom_timer);
	g_timeout_add (50, (GSourceFunc) zoom_timeout_callback, browser);
}

/** 
 * drag_drop:
 */
gboolean
drag_drop(GtkWidget * widget, GdkDragContext *context, gint x, gint y,
	  GtkSelectionData * selection_data, guint info, guint time)
{
	return FALSE;
}

/** 
 * drag_data_received:
*/
void
drag_data_received(GtkWidget * widget, GdkDragContext * context,
                  gint x, gint y,
                  GtkSelectionData * selection_data,
                  guint info, guint time)
{
	switch(info)
	{
	case DND_TARGET_NETSCAPE_URL:
	case DND_TARGET_GALEON_URL:
	case DND_TARGET_STRING:
		/* FIXME: should use browser_load_url here */
		gtk_moz_embed_load_url(GTK_MOZ_EMBED(widget), selection_data->data);
		break;
	default:
		break;
	}
}

#ifdef CONTEXT_TIMER_MENU
#define CMENU_TIMEOUT
#endif

#ifdef CMENU_HACK
#define CMENU_TIMEOUT
#endif
/**
 * cmenu_timeout_callback: display the links contextual menu
 */

#ifdef CMENU_TIMEOUT
gint 
cmenu_timeout_callback (GaleonBrowser *browser)
{
	GdkModifierType mask = 0;

#ifdef CMENU_HACK
	gint x, y;

	gdk_window_get_pointer (browser->WMain->window, &x, &y, &mask);

#endif

	g_return_val_if_fail (browser->cmenu_timer != NULL, FALSE);

	if (
#ifdef CMENU_HACK
		(mask & GDK_BUTTON3_MASK)
#endif
#ifdef CONTEXT_TIMER_MENU
/* disabling this portion of the context menu to avoid conflicts with
 * the right-click hack.  i left all the other code there because it's
 * used for both -- Nate */
/* now this should work agin - Ricardo */
#ifdef CMENU_HACK
		||
#endif
		(g_timer_elapsed (browser->cmenu_timer, NULL) >= browser->cmenu_time 
		 && gnome_config_get_int(CONF_GENERAL_LINKS_AUTOSHOW)==0)
#endif
		)
{

	context_show_link_menu(browser, browser->link);
#ifdef CONTEXT_TIMER_MENU
		return FALSE;
#endif
	} else if (mask & GDK_BUTTON2_MASK) {
		/* Avoid opening a bunch of windows when the user 
		   keeps the button pressed */
		static gchar *last_link = NULL;
		if (last_link != browser->link) {
			browser_create_from_url (browser->link);
			last_link = browser->link;
		}
	}

	return TRUE;
}
#endif

/**
 * Sets the view source mode for the current window
 */
void 
on_view_source_mode_activate (GtkCheckMenuItem *menuitem, GaleonBrowser *browser)
{
	mozilla_view_source (browser->mozEmbed, menuitem->active);
}

/** 
 * on_show_temporary_bookmarks_activate: show temp bookmarks window
 */
void
on_show_temporary_bookmarks_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	browser_show_temporary_bookmarks (browser);
}

void
on_add_temporary_bookmark_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	browser_add_temporary_bookmark (browser);
}

/** 
 * on_view_toolbar_activate: hide/show the toolbar
 */
void on_view_toolbar_activate (GtkCheckMenuItem *menuitem, GaleonBrowser *browser)
{
	if (menuitem->active) 
	{		
		browser_toolbar_show (browser);

		/* uncheck full screen menu */
		gtk_signal_handler_block_by_func(GTK_OBJECT(browser->view_fullscreen),
							    on_view_fullscreen_activate, browser);

		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(browser->view_fullscreen), FALSE);		

		gtk_signal_handler_unblock_by_func(GTK_OBJECT(browser->view_fullscreen),
						   on_view_fullscreen_activate, browser);
	}
	else
	{
		browser_toolbar_hide (browser);
	}
}

/** 
 * on_view_menubar_activate: hide/show the menubar
 */
void on_view_menubar_activate (GtkCheckMenuItem *menuitem, GaleonBrowser *browser)
{
	if (menuitem->active) 
	{
		browser_menubar_show (browser);

		/* uncheck full screen menu */
		gtk_signal_handler_block_by_func(GTK_OBJECT(browser->view_fullscreen),
							    on_view_fullscreen_activate, browser);

		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(browser->view_fullscreen), FALSE);		

		gtk_signal_handler_unblock_by_func(GTK_OBJECT(browser->view_fullscreen),
						   on_view_fullscreen_activate, browser);

	}
	else
	{
		browser_menubar_hide (browser);
	}
}

/** 
 * on_view_statusbar_activate: hide/show the menubar
 */
void on_view_statusbar_activate (GtkCheckMenuItem *menuitem, GaleonBrowser *browser)
{
	if (menuitem->active) 
	{
		browser_statusbar_show (browser);

		/* uncheck full screen menu */
		gtk_signal_handler_block_by_func(GTK_OBJECT(browser->view_fullscreen),
							    on_view_fullscreen_activate, browser);

		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(browser->view_fullscreen), FALSE);		

		gtk_signal_handler_unblock_by_func(GTK_OBJECT(browser->view_fullscreen),
						   on_view_fullscreen_activate, browser);

	}
	else
	{
		browser_statusbar_hide (browser);
	}
}

/** 
 * on_view_fullscreen_activate: set fullscreen mode
 */
void on_view_fullscreen_activate (GtkCheckMenuItem *menuitem, GaleonBrowser *browser)
{
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(browser->view_menubar), !menuitem->active);
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(browser->view_toolbar), !menuitem->active);
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(browser->view_statusbar), !menuitem->active);
}

void
on_location_drag_data_get (GtkWidget *widget, GdkDragContext *context,
			   GtkSelectionData *selection_data, 
			   guint info, guint time, 
			   GaleonBrowser *browser)
{
	gchar *url = gtk_editable_get_chars 
		(GTK_EDITABLE (browser->entry1), 0, -1);
	gchar *name = g_strdup (gtk_moz_embed_get_title (browser->mozEmbed));
	BookMarkItem *b;
	gchar *mem;
	if (strlen (name) == 0) name = g_strdup (_("Untitled"));
	switch (info) {
	case DND_TARGET_GALEON_BOOKMARK:
		b = bookmarks_new_bookmark (SITE, name, url, NULL, NULL);
		mem = bookmarks_item_to_string (b);
		gtk_selection_data_set 
			(selection_data, selection_data->target,
			 8, mem, strlen (mem));
		g_free (mem);
		break;
	case DND_TARGET_STRING:
	case DND_TARGET_NETSCAPE_URL:
	case DND_TARGET_GALEON_URL:
		gtk_selection_data_set 
			(selection_data, selection_data->target,
			 8, url, strlen (url));
				break;
	default:
		g_warning ("Unknown DND type");
		break;
	}
	g_free (url);
	g_free (name);
}

void
on_bookmarks_menuitem_drag_data_received (GtkWidget *widget, 
					  GdkDragContext *drag_context, gint x, gint y,
					  GtkSelectionData *selection_data, guint info,
					  guint time, GaleonBrowser *browser)
{
	BookMarkItem *b = NULL; 
	gchar *mem = selection_data->data;
	
	switch (info) {
	case DND_TARGET_GALEON_BOOKMARK:
		b = bookmarks_item_from_string (mem);
		break;
	case DND_TARGET_STRING:
	case DND_TARGET_NETSCAPE_URL:
	case DND_TARGET_GALEON_URL:
		b = bookmarks_new_bookmark (SITE, NULL, mem, NULL, NULL);
		break;
	default:
		g_warning ("Unknown DND type");
		break;
	}
	if (b) {
		GtkMenu *destination_menu = 
			bookmarks_create_copy_menu (bookmarks_editor, b);
		gnome_popup_menu_do_popup_modal
			(GTK_WIDGET (destination_menu), NULL, 
			 NULL, NULL, NULL);
		gtk_widget_destroy (GTK_WIDGET (destination_menu));
		bookmarks_save ();
		bookmarks_free_bookmark (b);
	}
}

void
on_url_entry_drag_data_received (GtkWidget *widget, 
				 GdkDragContext *drag_context, gint x, gint y,
				 GtkSelectionData *selection_data, guint info,
				 guint time, GaleonBrowser *browser)
{
	gchar *url = selection_data->data;
	gint i;
	
	switch (info) {
	case DND_TARGET_STRING:
	case DND_TARGET_NETSCAPE_URL:
	case DND_TARGET_GALEON_URL:
		gtk_editable_delete_text (browser->entry1, 0, -1);
		gtk_editable_insert_text (browser->entry1, url, 
					  strlen (url), &i);
		break;
	default:
		g_warning ("Unknown DND type");
		break;
	}
}

/* Ignore up/down key presses */
gboolean
on_zoom_spin_key_press_event (GtkWidget *widget, GdkEventKey *event, GaleonBrowser *browser)
{
	if ((event->keyval == GDK_Up) || (event->keyval == GDK_Down)) {
		event->keyval = GDK_VoidSymbol;
	}
	return FALSE;
}
