/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"

#define CONTEXT_MENU_ITEM(label, stock_id) \
 { GNOME_APP_UI_ITEM, label, NULL, NULL, NULL, NULL,  \
 GNOME_APP_PIXMAP_STOCK, stock_id, 0, 0, NULL}

static GnomeUIInfo contextual_link_menu_uiinfo[] =
{
  CONTEXT_MENU_ITEM(N_("Open"), GNOME_STOCK_MENU_OPEN),
  CONTEXT_MENU_ITEM(N_("Open in a new window"), GNOME_STOCK_MENU_NEW),
  CONTEXT_MENU_ITEM(N_("Download file"), GNOME_STOCK_MENU_SAVE),
  CONTEXT_MENU_ITEM(N_("Copy link location"), GNOME_STOCK_MENU_COPY),
  CONTEXT_MENU_ITEM(N_("Add temporary bookmark"), GNOME_STOCK_MENU_BLANK),
  GNOMEUIINFO_END
};

static GnomeUIInfo contextual_img_menu_uiinfo[] =
{
  CONTEXT_MENU_ITEM(N_("Open image"), GNOME_STOCK_MENU_OPEN),
  CONTEXT_MENU_ITEM(N_("Open image in a new window"), GNOME_STOCK_MENU_NEW),
  CONTEXT_MENU_ITEM(N_("Download image"), GNOME_STOCK_MENU_SAVE),
  GNOMEUIINFO_END
};

static GnomeUIInfo contextual_doc_menu_uiinfo[] =
{
  CONTEXT_MENU_ITEM(N_("Back"), GNOME_STOCK_MENU_BACK),
  CONTEXT_MENU_ITEM(N_("Forward"), GNOME_STOCK_MENU_FORWARD),
  CONTEXT_MENU_ITEM(N_("Reload"), GNOME_STOCK_MENU_REFRESH),
  GNOMEUIINFO_SEPARATOR,
  CONTEXT_MENU_ITEM(N_("View Source"), GNOME_STOCK_MENU_BLANK),
  CONTEXT_MENU_ITEM(N_("Save"), GNOME_STOCK_MENU_SAVE),
  CONTEXT_MENU_ITEM(N_("Find"), GNOME_STOCK_MENU_SEARCH),
  CONTEXT_MENU_ITEM(N_("Add temporary bookmark"), GNOME_STOCK_MENU_BLANK),
  GNOMEUIINFO_SEPARATOR,
  CONTEXT_MENU_ITEM(N_("Open frame in window"), GNOME_STOCK_MENU_OPEN),
  CONTEXT_MENU_ITEM(N_("Open frame in new window"), GNOME_STOCK_MENU_OPEN),
  CONTEXT_MENU_ITEM(N_("Reload frame"), GNOME_STOCK_MENU_REFRESH),
  CONTEXT_MENU_ITEM(N_("Add temporary bookmark for frame"), GNOME_STOCK_MENU_BLANK),
  GNOMEUIINFO_END
};

static GnomeUIInfo contextual_minimal_doc_menu_uiinfo[] =
{
  CONTEXT_MENU_ITEM(N_("View Source"), GNOME_STOCK_MENU_BLANK),
  CONTEXT_MENU_ITEM(N_("Save"), GNOME_STOCK_MENU_SAVE),
  CONTEXT_MENU_ITEM(N_("Find"), GNOME_STOCK_MENU_SEARCH),
  CONTEXT_MENU_ITEM(N_("Add temporary bookmark"), GNOME_STOCK_MENU_BLANK),
  GNOMEUIINFO_END
};
static GnomeUIInfo contextual_link_img_menu_uiinfo[] =
{
  CONTEXT_MENU_ITEM(N_("Open image"), GNOME_STOCK_MENU_OPEN),
  CONTEXT_MENU_ITEM(N_("Open image in a new window"), GNOME_STOCK_MENU_NEW),
  CONTEXT_MENU_ITEM(N_("Download image"), GNOME_STOCK_MENU_SAVE),
  GNOMEUIINFO_SEPARATOR,
  CONTEXT_MENU_ITEM(N_("Open"), GNOME_STOCK_MENU_OPEN),
  CONTEXT_MENU_ITEM(N_("Open in a new window"), GNOME_STOCK_MENU_NEW),
  CONTEXT_MENU_ITEM(N_("Download file"), GNOME_STOCK_MENU_SAVE),
  CONTEXT_MENU_ITEM(N_("Copy link location"), GNOME_STOCK_MENU_COPY),
  CONTEXT_MENU_ITEM(N_("Add temporary bookmark"), GNOME_STOCK_MENU_BLANK),
  GNOMEUIINFO_END
};

/**
 * context_show_link_menu: show a popup menu when a link is clicked
 **/
void context_show_link_menu (GaleonBrowser *browser, char *href)
{
	int selected;
	GtkWidget *popup;

	popup = gnome_popup_menu_new (contextual_link_menu_uiinfo);
	selected = gnome_popup_menu_do_popup_modal (popup, NULL, NULL,
							    NULL , NULL);

	switch (selected) {
	case 0: /* open */
		browser_load_url(browser, href);
		break;
	case 1: /* open in new window */
		browser_create_from_url (href);
		break;	
	case 2: /* save */
		save_url (browser->link);
		break;
	case 3: /* copy link */
		copy_to_clipboard (browser->link);
		break;
	case 4: /* add temp bookmark */
		add_temp_bookmark (SITE, NULL, browser->link, NULL);
		break;
	default:
		break;
	}

	gtk_widget_destroy(popup);
}

/**
 * context_show_doc_menu: show a popup menu when a document is clicked
 **/
void context_show_doc_menu (GaleonBrowser *browser, gpointer docshell)
{
    	int selected;
	GtkWidget *popup;
	gchar *name, *url;

	popup = gnome_popup_menu_new (contextual_doc_menu_uiinfo);
	selected = gnome_popup_menu_do_popup_modal (popup, NULL, NULL,
							    NULL , NULL);
	switch (selected) {
	case 0: /* back */
		gtk_moz_embed_go_back (browser->mozEmbed);
		break;
	case 1: /* forward */
		gtk_moz_embed_go_forward (browser->mozEmbed);
		break;
	case 2: /* reload */
		gtk_moz_embed_reload(browser->mozEmbed, GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
		break;
	case 4: /* view source */
		browser_view_source (browser, docshell);
		break;
	case 5: /* save */
		browser_show_save_dialog (browser, docshell);
		break;
	case 6: /* find */
		browser_show_find_dialog (browser, docshell);
		break;
	case 7: /* Add temp bookmark */
		name = g_strdup (gtk_moz_embed_get_title (browser->mozEmbed));
		url = gtk_editable_get_chars (GTK_EDITABLE (browser->entry1), 0, -1);
		if ( !name || strlen (name) == 0) {
			if (name) g_free (name);
			name = g_strdup (url);
		}
		add_temp_bookmark (SITE, name, url, NULL);
		g_free (url);
		g_free (name);
		break;
	case 9: /* open frame */
		browser_open_frame (browser, docshell, TRUE);
		break;
	case 10: /* open frame in new window */
		browser_open_frame (browser, docshell, FALSE);
		break;
	case 11: /* reload frame */
		mozilla_reload_docshell (docshell);
		break;
	case 12: /* add temp bookmark for frame */
		url = mozilla_get_docshell_url (docshell);
		if (url) {
			add_temp_bookmark (SITE, NULL, url, NULL);
			g_free (url);
		}
		break;
	default:
		break;
	}

	gtk_widget_destroy(popup);
}

/**
 * context_show_doc_menu: show a popup menu when a document is clicked
 **/
void context_show_minimal_doc_menu (GaleonBrowser *browser, gpointer docshell)
{
    	int selected;
	GtkWidget *popup;
	gchar *name, *url;

	popup = gnome_popup_menu_new (contextual_minimal_doc_menu_uiinfo);
	selected = gnome_popup_menu_do_popup_modal (popup, NULL, NULL,
							    NULL , NULL);
	switch (selected) {
	case 0: /* view source */
		browser_view_source (browser, docshell);
		break;
	case 1: /* save */
		browser_show_save_dialog (browser, docshell);
		break;
	case 2: /* find */
		browser_show_find_dialog (browser, docshell);
		break;
	case 3: /* Add temp bookmark */
		name = g_strdup (gtk_moz_embed_get_title (browser->mozEmbed));
		url = gtk_editable_get_chars (GTK_EDITABLE (browser->entry1), 0, -1);
		if ( !name || strlen (name) == 0) {
			if (name) g_free (name);
			name = g_strdup (url);
		}
		add_temp_bookmark (SITE, name, url, NULL);
		g_free (url);
		g_free (name);
		break;
	default:
		break;
	}

	gtk_widget_destroy(popup);
}

void context_show_img_menu (GaleonBrowser *browser, char *img)
{
	int selected;
	GtkWidget *popup;

	popup = gnome_popup_menu_new (contextual_img_menu_uiinfo);
	selected = gnome_popup_menu_do_popup_modal (popup, NULL, NULL,
							    NULL , NULL);

	switch (selected) {
	case 0: /* open image */
		browser_load_url(browser, img);
		break;
	case 1: /* open image in new window */
		browser_create_from_url (img);
		break;	
	case 2: /* save image */
		save_url (img);
		break;
	default:
		break;
	}

	gtk_widget_destroy(popup);
}

void context_show_link_img_menu (GaleonBrowser *browser, char *img, char *href)
{
	int selected;
	GtkWidget *popup;

	popup = gnome_popup_menu_new (contextual_link_img_menu_uiinfo);
	selected = gnome_popup_menu_do_popup_modal (popup, NULL, NULL,
							    NULL , NULL);

	switch (selected) {
	case 0: /* open image */
		browser_load_url(browser, img);
		break;
	case 1: /* open image in new window */
		browser_create_from_url (img);
		break;	
	case 2: /* save image */
		save_url (img);
		break;
	case 4: /* open */
		browser_load_url(browser, href);
		break;
	case 5: /* open in new window */
		browser_create_from_url (href);
		break;	
	case 6: /* save */
		save_url (href);
		break;
	case 7: /* copy link */
		copy_to_clipboard (href);
		break;
	case 8: /* add temp bookmark */
		add_temp_bookmark (SITE, NULL, href, NULL);
		break;
	default:
		break;
	}

	gtk_widget_destroy(popup);
}

