/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"

void on_find1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_BNext_clicked (GtkButton *button, GaleonBrowser *browser);
void on_BPrev_clicked (GtkButton *button, GaleonBrowser *browser);
void on_BClose_clicked (GtkButton *button, GaleonBrowser *browser);
void on_case_check_toggled (GtkToggleButton *togglebutton, GaleonBrowser *browser);
void on_find_entry_activate (GtkEditable *editable, GaleonBrowser *browser);
void on_find_entry_changed  (GtkEditable *editable, GaleonBrowser *browser);


/**
 * on_find1_activate: show find dialog
 */
void
on_find1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	browser_show_find_dialog (browser, NULL);
}

/**
 * on_BNext_clicked: next button clicked
 */
void on_BNext_clicked (GtkButton *button, GaleonBrowser *browser) 
{
	GtkEditable *editable = GTK_EDITABLE(glade_lookup_widget(GTK_WIDGET(button),"find_entry"));  
	GtkWidget *dialog = GTK_WIDGET(glade_lookup_widget(GTK_WIDGET(button),"dFind"));  
	GnomeEntry *find_gnome_entry = GNOME_ENTRY(glade_lookup_widget(GTK_WIDGET(button),"find_gnome_entry"));  
	GtkToggleButton *case_check = GTK_TOGGLE_BUTTON(glade_lookup_widget(GTK_WIDGET(button),"case_check"));  
	gpointer docshell;

	gchar *exp = gtk_editable_get_chars(editable, 0, -1);

	if (browser->start_find)
		gnome_entry_append_history(find_gnome_entry,FALSE,exp);

	docshell = gtk_object_get_data (GTK_OBJECT(dialog),"docshell");

	if (!mozilla_find(browser->mozEmbed, exp, gtk_toggle_button_get_active(case_check), FALSE,browser->start_find, docshell)) 
	{
		browser->start_find = TRUE;
		gnome_error_dialog(_("Not found"));
	}
	else 
	{
		browser->start_find = FALSE;
	}
}

/**
 * on_BPrev_clicked: previous button clicked
 */
void
on_BPrev_clicked (GtkButton *button, GaleonBrowser *browser)
{
	GtkEditable *editable = GTK_EDITABLE(glade_lookup_widget(GTK_WIDGET(button),"find_entry"));  
	GtkToggleButton *case_check = GTK_TOGGLE_BUTTON(glade_lookup_widget(GTK_WIDGET(button),"case_check"));  
	GnomeEntry *find_gnome_entry = GNOME_ENTRY(glade_lookup_widget(GTK_WIDGET(button),"find_gnome_entry"));  

	gchar *exp = gtk_editable_get_chars(editable, 0, -1);

	if (browser->start_find)
		gnome_entry_append_history(find_gnome_entry,FALSE,exp);

	if (!mozilla_find(browser->mozEmbed,exp,gtk_toggle_button_get_active(case_check),TRUE,browser->start_find, NULL))
	{
		browser->start_find = TRUE;
		gnome_error_dialog(_("Not found"));
	}
	else 
	{
		browser->start_find = FALSE;
	}

}

/**
 * on_BClose_clicked: close button clicked
 */
void
on_BClose_clicked (GtkButton *button, GaleonBrowser *browser) {
	GtkWidget *dfind = glade_lookup_widget(GTK_WIDGET(button),"dFind");  
	gtk_widget_destroy(dfind);
}

/**
 * on_case_check_toggled: case sensitive check button clicked
 */
void 
on_case_check_toggled (GtkToggleButton *togglebutton, GaleonBrowser *browser)
{
	browser->start_find = TRUE;
}

/**
 * on_find_entry_activate: entry activated
 */
void 
on_find_entry_activate (GtkEditable *editable, GaleonBrowser *browser)
{
	GtkToggleButton *case_check = GTK_TOGGLE_BUTTON(glade_lookup_widget(GTK_WIDGET(editable),"case_check"));  

	gchar *exp = gtk_editable_get_chars(editable, 0, -1);
	if (!mozilla_find(browser->mozEmbed,exp,gtk_toggle_button_get_active(case_check),FALSE,browser->start_find,NULL))
	{
		browser->start_find = TRUE;
		gnome_error_dialog(_("Not found"));
	}
	else 
	{
		browser->start_find = FALSE;
	}

}

/**
 * on_find_entry_changed: entry changed
 */
void
on_find_entry_changed  (GtkEditable *editable, GaleonBrowser *browser)
{
	browser->start_find = TRUE;
}

