/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>
#include <gmodule.h>
#include <glade/glade.h>
#include <gtkmozembed.h>

#include <libgnorba/gnorba.h>
#include "Galeon.h"
#include "Gtm.h"

#include <gnome-xml/xmlmemory.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>
#include <sys/stat.h>

/*
 * Some toplevel configuration options
 *
 * FIXME: most of these should either be deprecated or made into
 * run-time configuration options -- MattA
 */

#ifndef M17
#define M18
#endif

#define CORBA_INTERFACE       /* CORBA interface allowing remote startup   */
#ifndef M18
#define CONTEXT_TIMER_MENU    /* define to test context timer menu         */
#define CMENU_HACK            /* show menu if the right button is clicked  */
#endif 
#undef DEBUG_SIGNALS

/*
 * Configuration strings and their defaults... 
 */
#define CONF_GENERAL_HOMEPAGE         "/galeon/General/startpage=www.gnome.org"
#define CONF_GENERAL_HOMEPAGE_TYPE    "/galeon/General/startpage_type=0"
#define CONF_GENERAL_FIRSTPAGE_TYPE   "/galeon/General/firstpage_type=0"
#define CONF_GENERAL_NEWPAGE_TYPE     "/galeon/General/newpage_type=0"
#define CONF_GENERAL_BOOKMARK_TITLE   "/galeon/General/bookmark_title=0"
#define CONF_GENERAL_LINKS_AUTOSHOW   "/galeon/General/links_autoshow=1"
#define CONF_GENERAL_AUTOSHOW_TIME    "/galeon/General/autoshow_time=2"
#define CONF_GENERAL_NEWWINDOWS    "/galeon/General/newwindows=0"
#define CONF_APPEARANCE_WINWIDTH      "/galeon/Appearance/winwidth=775"
#define CONF_APPEARANCE_WINHEIGHT     "/galeon/Appearance/winheight=575"
#define CONF_APPEARANCE_WINDOWS_TITLE "/galeon/Appearance/windows_title=Galeon - %s"
#define CONF_MIME                     "/galeon/mime/"
#define CONF_ADVANCED_HISTORY_EXPIRE  "/galeon/Advanced/history_expire=100"
#define CONF_HANDLERS_USE_EXTERNAL_SOURCE_VIEWER \
                                       "/galeon/Handlers/use_external_source_viewer=FALSE"
#define CONF_HANDLERS_EXTERNAL_SOURCE_VIEWER \
                                       "/galeon/Handlers/external_source_viewer=gnome-edit"
#define CONF_DIR_OPEN "/galeon/Directories/open=~/"
#define CONF_DIR_SAVE "/galeon/Directories/save=~/"

/* the virtual URL of "My Portal" feature */
#define MYPORTAL_URL    "about:myportal"

/* bookmark file locations - relative to home dir */
#define NETSCAPE_BOOKMARKS "/.netscape/bookmarks.html"
#define MOZILLA_BOOKMARKS "/.mozilla/default/bookmarks.html"
				    
/* Context menu types */
#define CONTEXT_NONE     0
#define CONTEXT_DEFAULT  1
#define CONTEXT_LINK     2
#define CONTEXT_IMAGE    4
#define CONTEXT_DOCUMENT 8
#define CONTEXT_INPUT    64
#define CONTEXT_OTHER    128

/* key pressed char codes */
#define DOM_VK_PAGE_UP 33
#define DOM_VK_PAGE_DOWN 34
#define DOM_VK_UP 38
#define DOM_VK_DOWN 40
#define DOM_VK_LEFT 65361
#define DOM_VK_RIGHT 65363
#define DOM_VK_HOME 36
#define DOM_VK_END 35
#define DOM_VK_N 110
#define DOM_VK_B 98
#define DOM_VK_A 97
#define DOM_VK_E 101
#define DOM_VK_F 102
#define DOM_VK_G 103
#define DOM_VK_H 104
#define DOM_VK_R 114
#define DOM_VK_S 115
#define DOM_VK_O 111
#define DOM_VK_W 119
#define DOM_VK_Q 113
#define DOM_VK_T 116
#define DOM_VK_V 118
#define DOM_VK_F3 114
#define DOM_VK_F6 117

#define ALT_KEY 1
#define CTRL_KEY 2
#define SHIFT_KEY 4
#define META_KEY 8
#define KEY_CODE 16

typedef enum { // type of behaviour when making a new bookmark
	PAGE_TITLE,
	ASK_TITLE
} BookMarkTitle;

typedef enum { // start site type
	HOME_PAGE,
	LAST_PAGE,
	BLANK_PAGE,
} StartPageType;

typedef enum { // home page type
	URL_HOMEPAGE,
	MYPORTAL_HOMEPAGE
} HomePageType;

typedef enum { // types of actions to take for opening files
	SAVE_TO_DISK,
	RUN_PROGRAM,
	ASK_ACTION
} MimeAction;

typedef enum { // types of actions to show errors
	SHOW_ON_STATUS,
	SHOW_ON_DIALOG
} ErrorsShow;

typedef enum { // how to handling resuming crashed sessions
	RESUME_NEVER,
	RESUME_ASK,
	RESUME_ALWAYS
} AutoResume;

typedef enum { // netscape bookmark item type
	BM_SITE,
	BM_FOLDER,
	BM_FOLDER_END,
	BM_SEPARATOR,
	BM_UNKNOWN
} ns_item_type; 

typedef enum { // external handling
	HN_NOT_REQUIRED,
	HN_REQUIRED,
	HN_REQUIRED_IF_NOT_MIME
} handlers_type; 

typedef struct {
	GtkWidget *WMain;
	GtkMozEmbed *mozEmbed;
	const char *statusMessage;
	char *link;
	int         loadPercent;
	int         bytesLoaded;
	int         maxBytesLoaded;
	char       *tempMessage;
	gboolean    active; /* is the browser active? (i.e. loading a page) */

	/* GUI components */

	GtkWidget  *spinner; /* the GnomeAnimator spinner */
	GtkWidget  *spinner_button; /* the button that contain the spinner */

	GtkWidget *BBack;
	GtkWidget *BForward;
	GtkWidget *BRefresh;
	GtkWidget *BStop;
	GtkWidget *BStart;
	GtkEditable *zoom_spin;
	GtkEditable *entry1; /* FIXME rename ? */
	GtkWidget *main_toolbar_gnomeentry;
	GtkWidget *location_toolbar_gnomeentry;
	GtkWidget *eUrl;
	GtkWidget *appbar1;
	GtkWidget *main_toolbar;
	GtkWidget *go_toolbar1;
	GtkWidget *go_toolbar2;
	GtkWidget *main_toolbar_hbox;
	GtkWidget *location_dockitem;
	GtkWidget *main_dockitem;
	GtkWidget *menubar;
	GtkWidget *location_label1;
	GtkWidget *location_label2;
	GtkWidget *view_source_mode;
	GtkWidget *view_toolbar;
	GtkWidget *view_statusbar;
	GtkWidget *view_menubar;
	GtkWidget *view_fullscreen;
	GtkWidget *drag_pixmap;

	/* list of toolbars */
	GList *bookmarks_toolbars; 
	GtkTooltips *bookmarks_tooltips;

	/* if TRUE find must be restarted */
	gboolean start_find;

	/* Contextual menu timer */
	GTimer *cmenu_timer;
	int cmenu_timeout_id;
	float cmenu_time;

} GaleonBrowser;

typedef struct {
	gchar *name;
	gchar **extensions; /* an array of extension strings */
	MimeAction action;
	gchar *program;
	gint row;
} MimeItem;

typedef struct {
	gchar *title;
	gchar *url;
	GTime first;
	GTime last;
	gint visits;
} history_item;


typedef enum { // bookmarkitem type
	CATEGORY,
	SITE,
	SEPARATOR
} BookMarkType;

typedef struct _BookMarkItem {
	BookMarkType type;
	char *name;
	char *url;
	char *nick; // nickname of the bookmark
	/** The list of children if type == CATEGORY */
	GList *list;
	/** To find it in the edit bookmarks dialog */
	GtkCTreeNode *tree_item; 
	/** Parent category */
	struct _BookMarkItem *parent; 
	/** If type == CATEGORY says if should we create a toolbar for this */
	gboolean create_toolbar; 
	/** optional coments about the bookmark */
	char *notes; 
	/** if this type == CATEGORY, remember if the editor item was expanded */
	gboolean expanded;
} BookMarkItem;

typedef struct {
	BookMarkItem *root_bookmark;

	GtkWidget *dialog;
	GtkWidget *ctree;
	GtkWidget *edit_frame;
	GtkWidget *name_entry;
	GtkWidget *url_entry;
	GtkWidget *nick_entry;
	GtkWidget *create_toolbar_toggle;
	GtkWidget *notes_text;

	BookMarkItem *selection;
	BookMarkItem *last_pressed; /* for dnd */
	gboolean lock_edit_controls;
	gboolean dirty;
	void (*save_function) (void);
      
	GaleonBrowser *browser;
} bookmarks_editor_controls;

enum bookmarks_dnd_target_type { 
	DND_TARGET_GALEON_BOOKMARK, 
	DND_TARGET_GALEON_URL,
	DND_TARGET_NETSCAPE_URL, 
	DND_TARGET_TEXT_URI_LIST,
	DND_TARGET_STRING
};

extern const GtkTargetEntry bookmarks_dnd_targets[];
extern const gint bookmarks_dnd_targets_num_items;

/** The history window */
extern GtkWidget *dHistory;
extern GtkCList *clHistory;

/** the global list of MimeItems */
extern GList *mime_types; /* FIXME: can this be made local?  MattA */

/** The root of the bookmarks tree */
extern BookMarkItem *bookmarks_root;

/** The bookmarks editor */
extern bookmarks_editor_controls *bookmarks_editor;

GtkWidget *prefs;

/** The temp bookmarks window */
extern bookmarks_editor_controls *temp_bookmarks_window;

/** The list of opened windows */
extern GSList *window_list;

/** Module with all the symbols of the program */
GModule *mod_self;

/*
 * Function prototypes. Together these form the entire internal
 * API of Galeon. Could do with being thinned out a little -- MattA
 */

/* browser.c */
GaleonBrowser *browser_create (void);
GaleonBrowser *browser_create_default (gboolean first_window);
GaleonBrowser *browser_create_from_url (const gchar *url);
GaleonBrowser *browser_create_from_url_view_source (const gchar *url);
void browser_load_url (GaleonBrowser *browser, const gchar *url);
void browser_update_temp_message(GaleonBrowser *browser, const char *message);
void browser_update_nav_buttons(GaleonBrowser *browser);
void browser_update_status_bar_text(GaleonBrowser *browser);
void browser_show_error(GaleonBrowser *browser, char *errormsg);
void browser_progress_clear(GaleonBrowser *browser);
gboolean browser_progress_action (GaleonBrowser *browser);
void browser_show_open_dialog(GaleonBrowser *browser);
void browser_show_openurl_dialog(GaleonBrowser *browser);
void browser_show_save_dialog(GaleonBrowser *browser, gpointer docshell);
void browser_show_history (GaleonBrowser *browser);
void browser_add_temporary_bookmark (GaleonBrowser *browser);
void browser_show_find_dialog (GaleonBrowser *browser, gpointer docshell);
void browser_show_temporary_bookmarks (GaleonBrowser *browser);
void browser_menubar_show (GaleonBrowser *browser);
void browser_menubar_hide (GaleonBrowser *browser);
void browser_edit_bookmarks (GaleonBrowser *browser);
void browser_view_source (GaleonBrowser *browser, gpointer docshell);
void browser_view_source_external (GaleonBrowser *browser, gpointer docshell);
void browser_toolbar_show (GaleonBrowser *browser);
void browser_toolbar_hide (GaleonBrowser *browser);
void browser_statusbar_show (GaleonBrowser *browser);
void browser_statusbar_hide (GaleonBrowser *browser);
void browser_close (GaleonBrowser *browser);
void browser_open_frame (GaleonBrowser *browser, gpointer docshell,
			 gboolean same_window);

/* mozcallbacks.c */
void new_window_cb(GtkMozEmbed *embed, GtkMozEmbed **retval,
		   guint chromemask, GaleonBrowser *browser);
void visibility_cb(GtkMozEmbed *embed, gboolean visibility,
		   GaleonBrowser *browser);
void destroy_brsr_cb(GtkMozEmbed *embed, GaleonBrowser *browser);
void destroy_cb(GtkMozEmbed *embed, GaleonBrowser *browser);
void location_changed_cb(GtkMozEmbed *embed, GaleonBrowser *browser);
void title_changed_cb(GtkMozEmbed *embed, GaleonBrowser *browser);
void load_started_cb(GtkMozEmbed *embed, GaleonBrowser *browser);
void load_finished_cb(GtkMozEmbed *embed, GaleonBrowser *browser);
void progress_change_cb(GtkMozEmbed *embed, gint cur, gint max,
			GaleonBrowser *browser);
void link_message_cb(GtkMozEmbed *embed, GaleonBrowser *browser);
void js_status_cb(GtkMozEmbed *embed, GaleonBrowser *browser);
gint open_uri_cb(GtkMozEmbed *embed, const char *uri, GaleonBrowser *browser);
void net_status_change_cb (GtkMozEmbed *embed, gint flags,
			   guint status, GaleonBrowser *browser);

#ifdef M18
gint dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event, GaleonBrowser *browser);
gint dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event, GaleonBrowser *browser);
gint dom_mouse_up_cb (GtkMozEmbed *embed, gpointer dom_event, GaleonBrowser *browser);
gint dom_mouse_over_cb (GtkMozEmbed *embed, gpointer dom_event, GaleonBrowser *browser);
gint dom_key_press_cb (GtkMozEmbed *embed, gpointer dom_event, GaleonBrowser *browser);
void size_to_cb (GtkMozEmbed *embed, gint width, gint height, GaleonBrowser *browser);
#endif

/* history.c */
void history_load (void);
history_item *history_visited (gchar *url, gchar *title);
void history_save (void);
gchar *history_get_last_url (void);
void history_add_recent_menu (history_item *hi, GnomeApp *WMain);

/* history_callbacks.c */
gboolean on_clHistory_button_press_event(GtkWidget *widget,
					 GdkEventButton *event,
					 gpointer user_data);
void on_clHistory_click_column(GtkCList *clist, gint column,
			       gpointer user_data);
gboolean on_dHistory_delete_event(GtkWidget *widget, GdkEvent *event, 
				  gpointer user_data);
void on_history_ok_button_clicked(GtkButton *button, gpointer user_data);
void on_recent_menuitem_activate(GtkMenuItem *menuitem, gpointer user_data);

/* bookmarks.c */

#define BOOKMARKS_MENU_MAX_LENGTH 40 /** Max length of a bookmark menu item */

BookMarkItem *bookmarks_new_bookmark (BookMarkType type, const char *name, 
				      const char *url, const char *nick, 
				      const char *notes);
BookMarkItem *bookmarks_copy_bookmark (BookMarkItem *b);
void bookmarks_free_bookmark (BookMarkItem *b);
void bookmarks_load (void);
void bookmarks_save (void);
void bookmarks_move_bookmark (bookmarks_editor_controls *controls,
			      BookMarkItem *b, int where);
void bookmarks_insert_bookmark (BookMarkItem *b, BookMarkItem *near);
void bookmarks_remove_recursively (BookMarkItem *b);
void bookmarks_string_request_callback (gchar *string, BookMarkItem *new);
void bookmarks_create_menu (GaleonBrowser *browser);
void bookmarks_update_menu (void);
BookMarkItem *add_temp_bookmark (BookMarkType type, const gchar *name,
				 const gchar *url, BookMarkItem *parent);
void temp_bookmarks_init (void);
void temp_bookmarks_save (void);
GSList *bookmarks_list_all_urls(BookMarkItem *bookmark);
GSList *bookmarks_list_all_folders(BookMarkItem *bookmark);
gchar *bookmarks_parse_nick (gchar *text, BookMarkItem *root);
void bookmarks_editor_place_tree_item (bookmarks_editor_controls *controls,
				       BookMarkItem *b);
void bookmarks_editor_update_tree_item (bookmarks_editor_controls *controls,
					BookMarkItem *b);
void bookmarks_editor_remove_tree_item (bookmarks_editor_controls *controls, BookMarkItem *b);
GtkMenu* bookmarks_create_copy_menu (bookmarks_editor_controls *destination_window, 
				     BookMarkItem *source);
void bookmarks_toolbar_set_visible (GaleonBrowser *browser, gboolean visible);
gchar *bookmarks_item_to_string (BookMarkItem *root);
BookMarkItem *bookmarks_item_from_string (char *string);

/* netscape.c */
gint netscape_import_bookmarks (GString *filename);
gint netscape_export_bookmarks (gchar *filename, BookMarkItem *root);
gboolean netscape_import_prefs (void);

/* prefs.c */
void preferences_load(void);
void preferences_save(void);
/* FIXME... these are misnamed and poorly encapsulated */
void preferences_process(GtkWidget *widget);
void preferences_fill(GtkWidget *widget);
void preferences_mime_update(void);
void set_toolbar_style_window(GaleonBrowser *window); /* FIXME: move */
void set_toolbar_style_global(void); /* FIXME: move */

/* mime.c */
void mime_add(gchar *name, gchar *program, gchar *extensions,
	      MimeAction action);
void mime_delete(MimeItem *m);
void mime_update(MimeItem *m, gchar *name, gchar *program, 
		 gchar *extensions, MimeAction action);
void mime_get_entry(gchar **name, gchar **program, gchar **extensions,
		    MimeAction *action);

/* mozilla.cpp */
extern gboolean mozilla_preference_set(const char *preference_name, const char *new_value);
extern gboolean mozilla_preference_set_boolean (const char *preference_name, gboolean new_boolean_value);
extern gboolean mozilla_preference_set_int (const char *preference_name,int new_int_value);
extern gboolean mozilla_save (GtkMozEmbed *b,const char *fname, gpointer docshell);
extern gboolean mozilla_find (GtkMozEmbed *b,const char *exp,gboolean matchcase,gboolean search_backwards,gboolean start, gpointer docshell);
extern gboolean mozilla_sesion_history (GtkMozEmbed *b, char **titles[],
					int *count, int *index);

extern gboolean mozilla_sesion_history_go (GtkMozEmbed *b, int index);

extern gboolean mozilla_set_zoom (GtkMozEmbed *b, float z);
extern gboolean mozilla_print (GtkMozEmbed *b);
extern gboolean mozilla_view_source (GtkMozEmbed *b, gboolean view_source);
extern char* mozilla_get_docshell_url (gpointer ds);
extern gboolean mozilla_scroll (GtkMozEmbed *b, gint lines, gint pages);
extern gboolean mozilla_scroll_begin (GtkMozEmbed *b);
extern gboolean mozilla_scroll_end (GtkMozEmbed *b);

#ifdef M18
extern gint mozilla_get_mouse_button (gpointer event);
extern glong mozilla_get_context_menu_type(GtkMozEmbed *b, gpointer event, gchar **img, gpointer *docshell, gchar **link);
extern glong mozilla_get_key_pressed(gpointer event, int *modifier);
#endif
extern gboolean mozilla_reload_docshell (gpointer docshell);

/* session.h */
void session_autosave(void);
void session_autoresume(void);

/* browser_callbacks.c */
gboolean drag_drop(GtkWidget * widget, GdkDragContext *context, gint x, gint y, GtkSelectionData * selection_data, guint info, guint time);
void drag_data_received(GtkWidget *widget, GdkDragContext *context,
			gint x, gint y, GtkSelectionData *selection_data,
			guint info, guint time);
gint cmenu_timeout_callback (GaleonBrowser *browser);
void bookmark_import_netscape_style(GaleonBrowser * browser, GString * fname);
void on_bookmark_import_netscape(GtkMenuItem *menuitem,GaleonBrowser *browser);
void on_bookmark_import_mozilla(GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_bookmark_import_other(GtkMenuItem *menuitem, GaleonBrowser *browser);

void on_bookmark_file_ok_button_clicked(GtkButton *button, GaleonBrowser *browser);
void on_bookmark_file_cancel_button_clicked(GtkButton *button, GaleonBrowser *browser);
void on_open_url_ok_cb    (gchar *string, GaleonBrowser *browser);

/* misc.c */
void save_url(const gchar* url);
void save_url_with_gtm (const gchar* url);
void ask_dir(const gchar *url);
gboolean handle_foreign_protocols (const char *url);
gboolean handle_foreign_mime_types (const char *url);
GtkWidget *glade_lookup_widget(GtkWidget *widget, const gchar *widget_name);
gchar *user_file(gchar *fname, gint critical);
inline gchar *glade_file();
int g_str_is_url(gchar * string);
int file_exists(char *fname);
gchar *init_cache_dir(void);
GtkMenu *create_back_menu (GaleonBrowser *b);
GtkMenu *create_forward_menu (GaleonBrowser *b);
void menu_position_under_widget(GtkMenu *menu, gint *x, gint *y,
				gpointer user_data);
gboolean require_external_handler(const char *uri);
gboolean process_mime(const char *uri);
void glade_signal_connect_func(const gchar *cb_name, GtkObject *obj, 
			       const gchar *signal_name,
			       const gchar *signal_data, 
			       GtkObject *conn_obj, gboolean conn_after, 
			       gpointer user_data);
char *tmpnam_ex(char *suffix);
void copy_to_clipboard(const gchar *text);
void galeon_exit(void);
void remove_directory(const char *dirname, gboolean just_clean);

/* misc_callbacks.c */
void on_spinner_button_clicked(GtkButton *button, GaleonBrowser *browser);
void menuitem_bf_activated (GtkMenuItem *menuitem, gpointer user_data);
void on_browser_destroy (GtkObject *object, GaleonBrowser *browser);

/* bookmarks_callbacks.c */
void on_bookmark_add (GtkMenuItem *menuitem, BookMarkItem *parent);
void on_bookmark_activate  (GtkMenuItem *menuitem, gpointer user_data);
void on_bookmark_copy (GtkMenuItem *menuitem, BookMarkItem *parent);

/* auto_completion.c */
extern GList *posible_completions; /* FIXME */
void auto_completion_add_url(gchar *url);
void auto_completion_reset(void);
gchar *auto_completion_complete_url(gchar *prefix);
gchar *auto_completion_complete_url_extended(gchar *prefix);
void auto_completion_display_alternatives(GtkWidget *w);
void auto_completion_add_from_entry(GnomeEntry *e);

/* gtm.c */
gboolean gtm_add_url (const gchar *url, gchar *dir, gboolean disable_proxy,
		      gboolean disable_auto_dl);

/* spinner.c */
void spinner_create(GaleonBrowser *gb);
void spinner_destroy(GaleonBrowser *gb);

/* portal.c */
void portal_render_into_browser(GaleonBrowser *gb);

/* mozilla_prefs.c */
void mozilla_prefs_set (void);
void mozilla_prefs_set_default_charset (void);
void mozilla_prefs_set_languages (void);

/* context.c */
void context_show_img_menu (GaleonBrowser *browser, char *img);
void context_show_link_menu (GaleonBrowser *browser, char *href);
void context_show_link_img_menu (GaleonBrowser *browser, char *img, char *href);
void context_show_doc_menu (GaleonBrowser *browser, gpointer docshell);
void context_show_minimal_doc_menu (GaleonBrowser *browser, gpointer docshell);
