/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>
#include "gtkmozembed.h"
#include "galeon.h"

/**
 * Loads the selected row in the browser
 */
gboolean
on_clHistory_button_press_event (GtkWidget *widget, GdkEventButton *event,
				 gpointer user_data)
{
        gint row, column;
        gtk_clist_get_selection_info (clHistory, event->x, event->y, 
				      &row, &column);
        if (event->button != 1) {
                gtk_clist_select_row (clHistory, row, column);
        };
        if (event->button == 2 
	    || event->type == GDK_2BUTTON_PRESS) {
                GList *sel = clHistory->selection;
                if (sel != NULL) {
                        int row = GPOINTER_TO_INT (sel->data);
                        history_item *hi = gtk_clist_get_row_data (clHistory, row);
                        GtkWidget *WMain = GTK_WIDGET (
				gtk_object_get_data (GTK_OBJECT (dHistory), "WINDOW"));
                        GaleonBrowser *browser = 
				gtk_object_get_data (GTK_OBJECT(WMain),"mozilla");
			browser_load_url(browser, hi->url);
                }
        }
        return FALSE;
}

/**
 * Changes the ordering of the list
 */
void 
on_clHistory_click_column (GtkCList *clist, gint column,
			   gpointer user_data)
{
        if (column == clist->sort_column) {
                if (clist->sort_type == GTK_SORT_ASCENDING) {
                        gtk_clist_set_sort_type (clHistory, GTK_SORT_DESCENDING);
                } else {
                        gtk_clist_set_sort_type (clHistory, GTK_SORT_ASCENDING);
                }
        } else {
                gtk_clist_set_sort_column (clHistory, column);
                gtk_clist_set_sort_type (clHistory, GTK_SORT_ASCENDING);
        }
        gtk_clist_sort(clHistory);  // FIXME: should not be needed...
}


/**
 * Hides the window
 */
gboolean
on_dHistory_delete_event (GtkWidget *widget, GdkEvent *event,
			   gpointer user_data)
{
        gtk_widget_hide (dHistory);
        return TRUE;
}

/**
 * Hides the window
 */
void
on_history_ok_button_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
        GtkWidget *dHistory = glade_lookup_widget(GTK_WIDGET(button),"dHistory");
        gtk_widget_hide(dHistory);
}

/**
 * Goes to the url of the item
 */
void 
on_recent_menuitem_activate (GtkMenuItem *menuitem, gpointer user_data)
{
        GtkWidget *WMain = glade_lookup_widget (GTK_WIDGET (menuitem), "WMain");
        GaleonBrowser *browser = (GaleonBrowser*) gtk_object_get_data
		(GTK_OBJECT (WMain), "mozilla");
        history_item *hi = gtk_object_get_user_data (GTK_OBJECT (menuitem));

	browser_load_url(browser, hi->url);
}

