/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"

void on_about1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_ok_button1_clicked (GtkButton *button, GaleonBrowser *browser);
void on_cancel_button1_clicked (GtkButton *button, GaleonBrowser *browser);
void on_toolbar1_size_allocate(GtkWidget *widget, GtkAllocation *allocation, GaleonBrowser *browser);
void on_fssave_cancel_button_clicked(GtkButton *button, GaleonBrowser *browser);
void on_fssave_ok_button_clicked(GtkButton *button, GaleonBrowser *browser);
void on_view_source1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_mHistory_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
gboolean on_WMain_key_press_event (GtkWidget *widget, GdkEventKey *event,
			       GaleonBrowser *browser);
gboolean on_WMain_delete (GtkWidget *widget, GdkEventAny *event, GaleonBrowser *browser);

/**
 * on_about1_activate: show about dialog 
 */
void
on_about1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	static const char *author[] = {
		"Marco Pesenti Gritti <mpeseng@tin.it>",
		"Remco de Jong <rdj@rdj.cg.nu>",
		"Ricardo Fernandez Pascual <ric@users.sourceforge.net>",
		"Nate Case <nd@kracked.com>",
		"Bob Phan <ex-freek@yahoo.com>",
		"Matt Aubury <matt.aubury@embeddedsol.com>",
		"","Graphic arts :", "Toussaint Frederic (fred@linuxgraphic.org)",
		NULL
	};

	static GtkWidget * about;

	if (about) {
		gdk_window_show(about->window);
		gdk_window_raise(about->window);
		return;
	}
  
	about = gnome_about_new (_(PACKAGE), VERSION, _("Copyright (C) 2000 Marco Pesenti Gritti"),
				 author, _("A GNOME browser based on mozilla"),
				 SHARE_DIR"/logo.png");

	gtk_signal_connect (GTK_OBJECT (about), "destroy",GTK_SIGNAL_FUNC (gtk_widget_destroyed),&about);

	gtk_widget_show (about);
}

/**
 * on_WMain_delete: deleting toplevel window
 */
gboolean
on_WMain_delete (GtkWidget *widget, GdkEventAny *event, GaleonBrowser *browser)
{
	GnomeEntry *ge;
#ifdef DEBUG_SIGNALS
	g_print("WMain_delete\n");
#endif
	if (GNOME_IS_ANIMATOR (browser->spinner))
		gnome_animator_stop (GNOME_ANIMATOR(browser->spinner));

	ge = GNOME_ENTRY(browser->eUrl);
	gnome_entry_save_history(ge);

	gtk_widget_destroy(browser->WMain);

	return TRUE;
}

/**
 * on_browser_destroy: gtkmozembed component destroying
 */
void
on_browser_destroy (GtkObject *object, GaleonBrowser *browser)
{
#ifdef DEBUG_SIGNALS
	g_print("on_browser_destroy\n");
#endif
	window_list = g_slist_remove(window_list,browser);

	/* autosave the updated session */
	session_autosave();

	gnome_config_set_int("/galeon/Appearance/winwidth", browser->WMain->allocation.width);
	gnome_config_set_int("/galeon/Appearance/winheight", browser->WMain->allocation.height);

	if (browser->cmenu_timer) g_timer_destroy(browser->cmenu_timer);

	/* free the list of toolbars */
	g_list_free (browser->bookmarks_toolbars);

	/*scrub and free the GaleonBrowser structure. Temporarily removed: make galeon segfault on exit -- Marco */
	memset(browser, 0, sizeof(GaleonBrowser));
//	g_free(browser);

	if (g_slist_length(window_list) == 0)
	{
		galeon_exit();
	}
}

/**
 * on_ok_button1_clicked: file open dialog ok button clicked
 */
void
on_ok_button1_clicked (GtkButton *button, GaleonBrowser *browser)
{
	char * url;
	GtkWidget *fs = glade_lookup_widget(GTK_WIDGET(button), "fsopen");
	url = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
	gtk_entry_set_text(GTK_ENTRY(browser->entry1),url);
                                        
	browser_load_url(browser, url);
	
	gnome_config_set_string(CONF_DIR_OPEN,gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
	gnome_config_sync();
	
	gtk_widget_destroy(fs);
}

/**
 * on_cancel_button1_clicked: file open dialog cancel button clicked
 */
void
on_cancel_button1_clicked (GtkButton *button, GaleonBrowser *browser)
{
	GtkWidget *w = glade_lookup_widget(GTK_WIDGET(button), "fsopen");
	gtk_widget_destroy(w);
}

/** Go to a selected item in the back/forward history menu */
void menuitem_bf_activated (GtkMenuItem *menuitem, gpointer user_data)
{
	int index = GPOINTER_TO_INT (user_data);
	GaleonBrowser *browser = gtk_object_get_user_data (GTK_OBJECT (menuitem));
	mozilla_sesion_history_go (browser->mozEmbed, index);
}

/**
 * on_fssave_cancel_button_clicked: file save dialog cancel button clicked
 */
void 
on_fssave_cancel_button_clicked(GtkButton *button, GaleonBrowser *browser)
{
	GtkWidget *fs = glade_lookup_widget(GTK_WIDGET(button), "fssave");
	gtk_widget_destroy(fs);
}

/**
 * on_fssave_ok_button_clicked: file save dialog ok button clicked
 */
void 
on_fssave_ok_button_clicked(GtkButton *button, GaleonBrowser *browser)
{
	char * fname;
	gpointer docshell;
	GtkWidget *fs = glade_lookup_widget (GTK_WIDGET(button), "fssave");

	fname = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));

	docshell = gtk_object_get_data (GTK_OBJECT(fs), "docshell");

	if (!mozilla_save (browser->mozEmbed, fname, docshell))
		gnome_error_dialog(_("Cannot save the file"));

	gnome_config_set_string(CONF_DIR_SAVE,gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
	gnome_config_sync();

	gtk_widget_destroy(fs);
}

/**
 * on_spinner_button_clicked: spinner clicked
 */
void
on_spinner_button_clicked(GtkButton *button, GaleonBrowser *browser)
{
	/* FIXME: don't hardcode this */
	browser_load_url(browser, "http://galeon.sourceforge.net");
}

/**
 * on_view_source1_activate: view source
 */
void
on_view_source1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	browser_view_source (browser, NULL);
}

/**
 * on_mHistory_activate: show history dialog
 */
void on_mHistory_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	browser_show_history (browser);
}

