#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "gnome.h"
#include "gtkmozembed.h"
#include "gtkmozembed_internal.h"
#include "regex.h"

#include "nsIServiceManager.h"
#include "nsIPref.h"
#include "nsIDOMDocument.h"
#include "nsIDocShell.h"
#include "nsIDocShellTreeItem.h"
#include "nsIDocShellTreeOwner.h"
#include "nsIDocumentViewer.h"
#include "nsIWebBrowser.h"
#include "nsIDiskDocument.h"
#include "nsIDocument.h"
#include "nsFileSpec.h"
#include "nsCOMPtr.h"
#include "nsIWebNavigation.h"
#include "nsISHistory.h"
#include "nsISHEntry.h"
#include "nsIURI.h"
#include "nsITextServicesDocument.h"
#include "nsIPresShell.h"
#include "nsTextServicesCID.h"
#include "nsIContentViewer.h"
#include "nsIMarkupDocumentViewer.h"
#include "nsIContentViewerFile.h"
#include "nsITextScroll.h"
#include "nsIDOMMouseEvent.h"
#include "nsIDOMEventTarget.h"
#include "nsIDOMHTMLElement.h"
#include "nsIDOMNamedNodeMap.h"
#include "nsIURL.h"
#include "nsIScrollable.h"
#include "nsIFrame.h"
#include "nsIDocShellTreeNode.h"

/*FIXME: duplicate in galeon.h*/
#define CONTEXT_NONE     0
#define CONTEXT_DEFAULT  1
#define CONTEXT_LINK     2
#define CONTEXT_IMAGE    4
#define CONTEXT_DOCUMENT 8
#define CONTEXT_INPUT    64
#define CONTEXT_OTHER    128

#define ALT_KEY 1
#define CTRL_KEY 2
#define SHIFT_KEY 4
#define META_KEY 8
#define KEY_CODE 16

gboolean mozilla_find_setup(nsITextServicesDocument *aDoc, PRInt32 *outBlockOffset,
			    gboolean search_backwards, gboolean start);
gchar* mozilla_get_attribute (nsIDOMNode *node, gchar *attribute);
nsIDocShell* mozilla_get_primary_docshell (GtkMozEmbed *b);
nsIDocShell* mozilla_get_event_docshell (GtkMozEmbed *b, nsIDOMMouseEvent * event);
gboolean mozilla_resolve_url (char *base, char *relative, char **resolved);

static NS_DEFINE_CID(kCTextServicesDocumentCID, NS_TEXTSERVICESDOCUMENT_CID);
static NS_DEFINE_CID(kStandardURLCID, NS_STANDARDURL_CID);

#ifndef M17
#define M18
#endif

#ifdef M18
#define PREF_ID NS_PREF_CONTRACTID
#else
#define PREF_ID NS_PREF_PROGID
#endif

extern "C" gboolean
mozilla_preference_set(const char *preference_name, const char *new_value)
{
  g_return_val_if_fail (preference_name != NULL, FALSE);
  g_return_val_if_fail (new_value != NULL, FALSE);
  nsCOMPtr<nsIPref> pref = do_CreateInstance(PREF_ID);

  if (pref)
    {
      nsresult rv = pref->SetCharPref (preference_name, new_value);            
      return NS_SUCCEEDED (rv) ? TRUE : FALSE;
    }
  
  return FALSE;
}

extern "C" gboolean
mozilla_preference_set_boolean (const char        *preference_name,
				gboolean        new_boolean_value)
{
  g_return_val_if_fail (preference_name != NULL, FALSE);
  
  nsCOMPtr<nsIPref> pref = do_CreateInstance(PREF_ID);
  
  if (pref)
    {
      nsresult rv = pref->SetBoolPref (preference_name,
				       new_boolean_value ? PR_TRUE : PR_FALSE);
      
      return NS_SUCCEEDED (rv) ? TRUE : FALSE;
    }
  
  return FALSE;
}

extern "C" gboolean
mozilla_preference_set_int (const char        *preference_name,
				int        new_int_value)
{
  g_return_val_if_fail (preference_name != NULL, FALSE);
  
  nsCOMPtr<nsIPref> pref = do_CreateInstance(PREF_ID);
  
  if (pref)
    {
      nsresult rv = pref->SetIntPref (preference_name,
				       new_int_value);
      
      return NS_SUCCEEDED (rv) ? TRUE : FALSE;
    }
  
  return FALSE;
}

extern "C" gboolean
mozilla_save (GtkMozEmbed *b, const char *fname, nsIDocShell *docshell)
{
	nsresult result ;	
	nsCOMPtr<nsIDocShell> ds;

#ifdef M18
	if (docshell)
	{
		ds = docshell;
	}
	else 
	{
#endif
		ds = mozilla_get_primary_docshell (b);
		if (!ds) return FALSE;
#ifdef M18
	}
#endif

	/* get nsIPresShell */

	nsCOMPtr<nsIPresShell> presShell;
	ds->GetPresShell(getter_AddRefs(presShell));
	if (!presShell) return FALSE;

	/* get nsIDocument */

	nsCOMPtr<nsIDocument> Document;
	presShell->GetDocument(getter_AddRefs(Document));

	/* get nsIDOMDocument */

	nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(Document);
	if (!domDoc) return FALSE;

	
	/* write the document on the disk */

	nsString mime_type;

	mime_type.AssignWithConversion("text/html");

	nsCOMPtr<nsIDiskDocument> diskDoc = do_QueryInterface(domDoc);
	
	nsFileSpec fileSpec(fname, PR_FALSE);
	result = diskDoc->SaveFile(&fileSpec, PR_TRUE, PR_TRUE, mime_type, NULL, 0);
	if (!NS_SUCCEEDED(result)) return FALSE;

	/* save succeded */
	return TRUE;

}

/**
 * Gets the session history and current position of a GtkMozEmbed. On return, *titles 
 * will be an array of char * wich must be freed.
 */
extern "C" gboolean
mozilla_sesion_history (GtkMozEmbed *b, char **titles[], int *count, int *index)
{
	nsresult result;

	nsCOMPtr<nsIDocShell> ds;
	ds = mozilla_get_primary_docshell (b);
	if (!ds) return FALSE;

	nsCOMPtr<nsIWebNavigation> wn = do_QueryInterface (ds, &result);
	if (!wn) return FALSE;
	
	nsCOMPtr<nsISHistory> h;
	result = wn->GetSessionHistory (getter_AddRefs (h));
	if (!NS_SUCCEEDED (result) || (!h)) return FALSE;

	h->GetCount (count);
	h->GetIndex (index);

	char **t = g_new(char *, *count);
	nsCOMPtr<nsISHEntry> he;
	char *s;
	PRUnichar *title;
	for (PRInt32 i = 0; i < *count; i++) {

		result = h->GetEntryAtIndex (i, PR_FALSE, getter_AddRefs (he));
		if (!NS_SUCCEEDED(result) || (!he)) return FALSE;

		result = he->GetTitle (&title);
		if (!NS_SUCCEEDED(result) || (!title)) return FALSE;

		// The title is in 16-bit unicode, we should make it 8bit (UTF)
		// FIXME: Do this properly
		s = g_new (char, 100);
		for (int j = 0, k = 0; (s[j] = title[k]) && (j < 100); j++, k++) ;
		s[99] = 0;
		
		t[i] = s;

		// should I free title?
	}
	*titles = t;
	return TRUE;
}

/**
 * Goes to the SHEntry at the given index
 */
extern "C" gboolean
mozilla_sesion_history_go (GtkMozEmbed *b, int index)
{
	nsresult result;
	
	nsCOMPtr<nsIDocShell> ds;
	ds = mozilla_get_primary_docshell (b);
	if (!ds) return FALSE;

	nsCOMPtr<nsIWebNavigation> wn = do_QueryInterface(ds, &result);
	if (!wn) return FALSE;
	
	wn->GotoIndex (index);
	
	return TRUE;
}

/**
 * mozilla_find: find a word in the document
 */
extern "C" gboolean
mozilla_find (GtkMozEmbed *b,const char *exp,gboolean matchcase,gboolean search_backwards,gboolean start, nsIDocShell *docshell)
{
#ifdef GCC_COMPATIBILITY_HACK
	return 0;
#else
	nsresult result ;	
	nsCOMPtr<nsIDocShell> ds;

#ifdef M18
	if (docshell)
	{
		ds = docshell;
	}
	else 
	{
#endif
		ds = mozilla_get_primary_docshell (b);
		if (!ds) return FALSE;
#ifdef M18
	}
#endif M18

	/* get nsIPresShell */

	nsCOMPtr<nsIPresShell> presShell;
	result = ds->GetPresShell(getter_AddRefs(presShell));
	if (!NS_SUCCEEDED(result) || (!presShell)) return FALSE;

	/* get nsIDocument */

	nsCOMPtr<nsIDocument> document;
	result = presShell->GetDocument(getter_AddRefs(document));
	if (!NS_SUCCEEDED(result) || (!document)) return FALSE;

	/* get nsIDOMDocument */

	nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(document);
	if (!domDoc) return FALSE;

	/* create nsIServiceDocument */

	nsCOMPtr<nsITextServicesDocument> textDoc;
	result = nsComponentManager::CreateInstance(kCTextServicesDocumentCID,nsnull,
						    NS_GET_IID(nsITextServicesDocument),
						    getter_AddRefs(textDoc));
	if (NS_FAILED(result) || !textDoc) return FALSE;

	/* init nsIServiceDocument */

	result = textDoc->InitWithDocument(domDoc, presShell);
	if (NS_FAILED(result)) return FALSE; 

	
	PRInt32 outBlockOffset = -1;

	mozilla_find_setup(textDoc,&outBlockOffset,search_backwards,start);

	PRBool atExtremum = PR_FALSE;     

	regex_t preg;
	regmatch_t match[1];
	int res;
	int eflags = 0;
	
	if (!matchcase) eflags = REG_ICASE;

	regcomp(&preg,exp,eflags);

	while ( NS_SUCCEEDED(textDoc->IsDone(&atExtremum)) && !atExtremum )
	{
		nsString str;
		char *cstr, *tempcstr;

		result = textDoc->GetCurrentTextBlock(&str);
		if (NS_FAILED(result)) return result;

		cstr = str.ToNewCString();
		tempcstr = cstr;

		match[0].rm_so = -1;
		match[0].rm_eo = -1;

		if (!search_backwards)
		{
			PRInt32 offset = 0;
			if (outBlockOffset!=-1) offset = outBlockOffset;
			res = regexec(&preg,tempcstr+offset,1,match,0);

			if (res==0) {

				textDoc->SetSelection(match[0].rm_so+offset,match[0].rm_eo-match[0].rm_so);
				textDoc->ScrollSelectionIntoView();
				regfree(&preg);
				return true;
			}

		}
		else
		{
			gboolean not_found = TRUE;
			PRInt32 eo = 0;
			PRInt32 old_rm_eo = 0;
			if (outBlockOffset!=-1) cstr[outBlockOffset] = 0;
			while ((res = regexec(&preg,tempcstr,1,match,0))==0)
			{
				eo += old_rm_eo;
				old_rm_eo = match[0].rm_eo;
				tempcstr += match[0].rm_eo;
				not_found = FALSE;
			}
			
			if (!not_found) {

				textDoc->SetSelection(match[0].rm_so+eo,match[0].rm_eo-match[0].rm_so);
				textDoc->ScrollSelectionIntoView();
				regfree(&preg);
				return true;
			}
		}

		g_free(cstr);


		if (!search_backwards)
		{
			result = textDoc->NextBlock(); 
		}
		else
		{
			result = textDoc->PrevBlock();
		}

		if (NS_FAILED(result)) return FALSE;

		outBlockOffset = -1;
	}

	regfree(&preg);
	return FALSE;
#endif
}

/**
 * mozilla_find_setup: get the search initial position
 */
gboolean 
mozilla_find_setup(nsITextServicesDocument *aDoc, PRInt32 *outBlockOffset, gboolean search_backwards, gboolean start)
{
	nsresult rv;
	nsITextServicesDocument::TSDBlockSelectionStatus blockStatus;
	PRInt32 selOffset;
	PRInt32 selLength;

	rv = aDoc->LastSelectedBlock(&blockStatus, &selOffset, &selLength);

	if (!search_backwards)
	{
		if (NS_SUCCEEDED(rv) && (!start))
		{
			*outBlockOffset = selOffset+selLength;
		}
		else 
		{
			rv = aDoc->FirstBlock();
		}
	}
	else
	{
		if (NS_SUCCEEDED(rv) && (!start))
		{
			*outBlockOffset = selOffset;
		}
		else 
		{
			rv = aDoc->LastBlock();
		}
	
	}
	
	return TRUE;
}

/**
 * Sets the zoom factor of a browser
 */
extern "C" gboolean
mozilla_set_zoom (GtkMozEmbed *b, float f)
{
	nsresult result;

	nsCOMPtr<nsIDocShell> ds = mozilla_get_primary_docshell (b);
	if (!ds) return FALSE;

	nsCOMPtr<nsIContentViewer> cv;
	result = ds->GetContentViewer (getter_AddRefs (cv));
	if (!NS_SUCCEEDED (result) || !cv) return FALSE;
	
	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(cv, &result);
	if (!mdv) return FALSE;
	
	result = mdv->SetTextZoom (f);
	if (!NS_SUCCEEDED (result)) return FALSE;

	return TRUE;
}

/**
 * mozilla_print: print a document
 */
extern "C" gboolean
mozilla_print (GtkMozEmbed *b)
{
	/* FIXME why doesnt work ? a gtkmozembed issue ? mozilla-prefs ? */
	nsresult result;

	nsCOMPtr<nsIDocShell> ds = mozilla_get_primary_docshell (b);
	if (!ds) return FALSE;

	nsIContentViewer *pContentViewer = nsnull;
	result = ds->GetContentViewer(&pContentViewer);
	if (NS_SUCCEEDED(result) || !pContentViewer) return FALSE;
		
	nsCOMPtr<nsIContentViewerFile> spContentViewerFile = do_QueryInterface(pContentViewer);
	if (!spContentViewerFile) return FALSE;

	spContentViewerFile->Print(PR_TRUE, nsnull);
	NS_RELEASE(pContentViewer);

	return TRUE;
}

/**
 * Turn a browser source view mode
 */
extern "C" gboolean 
mozilla_view_source (GtkMozEmbed *b, gboolean view_source)
{
	nsresult result;

	nsCOMPtr<nsIDocShell> ds = mozilla_get_primary_docshell (b);
	if (!ds) return FALSE;

	if (view_source) {
		result = ds->SetViewMode (nsIDocShell::viewSource);
	} else {
		result = ds->SetViewMode (nsIDocShell::viewNormal);
	}
	
	if (!NS_SUCCEEDED (result)) return FALSE;

	return TRUE;
}


/**
 * mozilla_scroll_lines: scroll the current view by a number of lines
 * FIXME: this is probably all wrong, but it works and we need it! -- MattA
 */
extern "C" gboolean
mozilla_scroll (GtkMozEmbed *b, gint lines, gint pages)
{
	nsresult result;
	nsCOMPtr<nsIDocShell> ds;

	ds = mozilla_get_primary_docshell (b);
	if (!ds) return FALSE;

	nsCOMPtr<nsITextScroll> ts = do_QueryInterface(ds, &result);
	if (!ts) return FALSE;

	ts->ScrollByLines(lines);
	ts->ScrollByPages(pages);

	return TRUE;
}

extern "C" gboolean
mozilla_scroll_begin (GtkMozEmbed *b)
{
	nsresult result;
	nsCOMPtr<nsIDocShell> ds;

	ds = mozilla_get_primary_docshell (b);
	if (!ds) return FALSE;

	nsCOMPtr<nsIScrollable> scrollable = do_QueryInterface(ds, &result);
	if (!scrollable) return FALSE;

	/* the first 1 is the orientation (vertical) */
	scrollable->SetCurScrollPos (1, 0);
	
	return TRUE;
}

extern "C" gboolean
mozilla_scroll_end (GtkMozEmbed *b)
{
	nsresult result;
	nsCOMPtr<nsIDocShell> ds;

	ds = mozilla_get_primary_docshell (b);
	if (!ds) return FALSE;

	nsCOMPtr<nsIScrollable> scrollable = do_QueryInterface(ds, &result);
	if (!scrollable) return FALSE;

	PRInt32 min, max;
	/* the first 1 is the orientation (vertical) */
	scrollable->GetScrollRange (1, &min, &max);
	scrollable->SetCurScrollPos (1, max);
	
	return TRUE;
}

#ifdef M18

/**
 * mozilla_get_mouse_button: get pressed mouse button from DOM mouse event
 */
extern "C" gint 
mozilla_get_mouse_button (gpointer event)
{
	nsresult result;
	PRUint16 button;

	result = ((nsIDOMMouseEvent *)event)->GetButton(&button);
	if (!NS_SUCCEEDED (result)) return 0;

	return button;
}

/**
 * mozilla_get_key_pressed: get pressed key button from a DOM key event
 */
extern "C" glong 
mozilla_get_key_pressed (gpointer event, int *modifier)
{
	PRUint32 key;
	nsIDOMKeyEvent *keyevent = ((nsIDOMKeyEvent *)event);
	PRBool mod_key;

	*modifier = 0;

	keyevent->GetCharCode(&key);

	if (key==0) 
	{
		keyevent->GetKeyCode(&key);
		*modifier |= KEY_CODE;
	}

	keyevent->GetAltKey(&mod_key);
	if (mod_key) *modifier |= ALT_KEY;

	keyevent->GetShiftKey(&mod_key);
	if (mod_key) *modifier |= SHIFT_KEY;

	keyevent->GetMetaKey(&mod_key);
	if (mod_key) *modifier |= META_KEY;
	
	keyevent->GetCtrlKey(&mod_key);
	if (mod_key) *modifier |= CTRL_KEY;

	return key;
}

/**
 * mozilla_get_context_menu_type: get what context menu type need to be displayed
 */
extern "C" glong
mozilla_get_context_menu_type(GtkMozEmbed *b, gpointer event, gchar **img, gpointer *docshell, gchar **link)
{
	nsresult res;

	*img = *link = NULL;

	nsIDOMMouseEvent *aMouseEvent = 	(nsIDOMMouseEvent*)event;

	*docshell = mozilla_get_event_docshell (b, aMouseEvent);
	if (!docshell) return CONTEXT_NONE;

	nsCOMPtr<nsIDOMEventTarget>  targetNode;
	res = aMouseEvent->GetTarget(getter_AddRefs(targetNode));
	if (NS_FAILED(res))
	{
		return CONTEXT_NONE;
	}
	if (!targetNode)
	{
		return CONTEXT_NONE;
	}

	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(targetNode);
	if (!node)
	{
		return CONTEXT_NONE;
	}
	
	PRUint32 flags = CONTEXT_NONE;
	nsCOMPtr<nsIDOMHTMLElement> element;
	do {
		PRUint16 type;
		node->GetNodeType(&type);

		element = do_QueryInterface(node);
		if (element)
		{
			nsAutoString tag;
			element->GetTagName(tag);

			if (tag.EqualsWithConversion("input", PR_TRUE))
			{
				flags |= CONTEXT_INPUT;
			}
			else if (tag.EqualsWithConversion("img", PR_TRUE))
			{
				flags |= CONTEXT_IMAGE;

				char *src = mozilla_get_attribute (node, g_strdup("src"));
				if (!src) return CONTEXT_NONE;

				mozilla_resolve_url (gtk_moz_embed_get_location (b), src, img);
				if (!img) return CONTEXT_NONE;		     

			}
			else
			{
				flags |= CONTEXT_OTHER;
			}

			nsCOMPtr<nsIDOMNamedNodeMap> attributes;
			node->GetAttributes(getter_AddRefs(attributes));
			if (attributes)
			{
				nsCOMPtr<nsIDOMNode> hrefNode;
				nsAutoString href; href.AssignWithConversion("href");
				attributes->GetNamedItem(href, getter_AddRefs(hrefNode));
				if (hrefNode)
				{
					flags |= CONTEXT_LINK;

					char *href =  mozilla_get_attribute (node, g_strdup("href"));
					if (!href) return CONTEXT_NONE;

					mozilla_resolve_url (gtk_moz_embed_get_location (b), href, link);
					if (!link) return CONTEXT_NONE;
					
					break;
				}
			}
		}
		nsCOMPtr<nsIDOMNode> parentNode;
		node->GetParentNode(getter_AddRefs(parentNode));
		
		if (!parentNode)
		{
			node = nsnull;
			flags |= CONTEXT_DOCUMENT;
			break;
		}
		node = parentNode;
	} while (node);
	
	return flags;
}

/* Nautilus CREDITS here */
gchar* mozilla_get_attribute (nsIDOMNode *node, gchar *attribute)
{
	nsresult result;
	
	nsCOMPtr<nsIDOMNamedNodeMap> attributes;
	result = node->GetAttributes(getter_AddRefs (attributes));
	if (!NS_SUCCEEDED (result) || !attributes) return NULL;

	nsAutoString attr; 

	attr.AssignWithConversion (attribute);
                                        
	nsCOMPtr<nsIDOMNode> attrNode;
	result = attributes->GetNamedItem (attr, getter_AddRefs (attrNode));
	if (!NS_SUCCEEDED (result) || !attrNode) return NULL;

	nsAutoString nodeValue;
			
	result = attrNode->GetNodeValue (nodeValue);
	if (!NS_SUCCEEDED (result)) return NULL;
                                                
	char *cstr = nodeValue.ToNewCString();
	char *lifeSucks = g_strdup (cstr);
			
	nsMemory::Free (cstr);
	
	return lifeSucks;
}

nsIDocShell* mozilla_get_event_docshell (GtkMozEmbed *b, nsIDOMMouseEvent * event)
{
	nsresult result;

	nsCOMPtr<nsIDocShell> ds = mozilla_get_primary_docshell (b);
	if (!ds) return NULL;

	nsIDocShellTreeItem *child_sti;
	nsCOMPtr<nsIDocShellTreeNode> tn = do_QueryInterface(ds);
	if (!tn) return NULL;

	int n;
	tn->GetChildCount (&n);
	/* no frames. return primary docshell */
	if (n==0) return ds;
	
	for (int k = 0; k<n; k++)
	{
		tn->GetChildAt(k,&child_sti);
		
		nsCOMPtr<nsIDocShell> child_ds = do_QueryInterface(child_sti); 
		if (!child_ds) return NULL;

		nsCOMPtr <nsIContentViewer> cv;
		result = child_ds->GetContentViewer (getter_AddRefs(cv));
		if (!NS_SUCCEEDED (result) || !cv) return NULL;
		
		nsCOMPtr <nsIDOMDocument> dd;
		result = cv->GetDOMDocument (getter_AddRefs(dd));
		if (!NS_SUCCEEDED (result) || !dd) return NULL;

		nsCOMPtr<nsIDOMEventTarget>  targetNode;
		result = event->GetTarget(getter_AddRefs(targetNode));		
		if (!NS_SUCCEEDED (result) || !targetNode) return NULL;

		nsCOMPtr<nsIDOMNode> node = do_QueryInterface(targetNode);
		if (!node) return NULL;
		
		nsCOMPtr<nsIDOMDocument> dom_doc;
		result = node->GetOwnerDocument(getter_AddRefs(dom_doc));
		if (!NS_SUCCEEDED (result) || !dom_doc) return NULL;

		if (dom_doc == dd) return child_ds;
	}
	return NULL;
}


#endif
extern "C" char* 
mozilla_get_docshell_url (nsIDocShell *ds)
{
#ifdef M18
	nsresult rv;
	char *result;

	/* get nsIPresShell */

	nsCOMPtr<nsIPresShell> presShell;
	rv = ds->GetPresShell(getter_AddRefs(presShell));
	if (!NS_SUCCEEDED (rv) || !presShell) return FALSE;

	/* get nsIDocument */

	nsCOMPtr<nsIDocument> doc;
	rv = presShell->GetDocument(getter_AddRefs(doc));
	if (!NS_SUCCEEDED (rv) || !doc) return FALSE;	

	nsCOMPtr <nsIURI> url = doc->GetDocumentURL();
	if (!url) return NULL;

	url->GetSpec (&result);

	return result;
#else
	return NULL;
#endif
}

nsIDocShell* mozilla_get_primary_docshell (GtkMozEmbed *b)
{
	nsresult result;
	nsIWebBrowser *wb;
	gtk_moz_embed_get_nsIWebBrowser (b, &wb);

	nsCOMPtr<nsIDocShell> ds;
#ifndef M18
	result = wb->GetDocShell (getter_AddRefs (ds));
	if (!NS_SUCCEEDED (result)) return NULL;
#else
        nsCOMPtr<nsIDocShellTreeItem> browserAsItem = do_QueryInterface(wb);
	if (!browserAsItem) return NULL;

	// get the tree owner for that item
	nsCOMPtr<nsIDocShellTreeOwner> treeOwner;
	result = browserAsItem->GetTreeOwner(getter_AddRefs(treeOwner));
	if (!NS_SUCCEEDED (result) || ! treeOwner) return NULL;

	// get the primary content shell as an item
	nsCOMPtr<nsIDocShellTreeItem> contentItem;
	result = treeOwner->GetPrimaryContentShell(getter_AddRefs(contentItem));
	if (!NS_SUCCEEDED (result) || ! contentItem) return NULL;

	// QI that back to a docshell
	ds = do_QueryInterface(contentItem);
#endif

	return ds;
}

gboolean
mozilla_resolve_url (char *base, char *relative, char **resolved)
{
	nsresult result;
	nsIURL* url;

	result = nsComponentManager::CreateInstance (kStandardURLCID, nsnull,
						     NS_GET_IID(nsIURL),
						     (void**)&url);
	if (!NS_SUCCEEDED (result)) return FALSE;

	result = url->SetSpec (base);
	if (!NS_SUCCEEDED (result)) return FALSE;

	url->Resolve (relative, resolved);
	if (!NS_SUCCEEDED (result) || !(*resolved)) return FALSE;

	return TRUE;
}

extern "C" gboolean
mozilla_reload_docshell (gpointer docshell)
{
	nsresult result;
	nsIDocShell *ds = (nsIDocShell *) docshell;
	nsCOMPtr<nsIWebNavigation> wn = do_QueryInterface (ds, &result);
	if (!wn || !NS_SUCCEEDED (result)) return FALSE;
	result = wn->Reload (nsIWebNavigation::LOAD_FLAGS_BYPASS_CACHE | 
			     nsIWebNavigation::LOAD_FLAGS_BYPASS_PROXY);
	if (!NS_SUCCEEDED (result)) return FALSE;
	return TRUE;		
}
