#include "galeon.h"

static char *autodetect_charset_prefs[] =
{
	"",
	"zh_parallel_state_machine",
	"cjk_parallel_state_machine",
	"ja_parallel_state_machine",
	"ko_parallel_state_machine",
	"ruprob",
	"zhcn_parallel_state_machine",
	"zhtw_parallel_state_machine",
	"ukprob"
};

void mozilla_prefs_set_autodetect_charset (int charset);
void mozilla_prefs_set_default_charset ();
void mozilla_prefs_set_languages ();
char * get_bracketed_string (char *str);
/**
 * mozilla_prefs_set: set mozilla preferences
 * FIXME: hairball alert -- MattA
 */
void
mozilla_prefs_set(void)
{
	gchar *http_proxy, *ftp_proxy;
	int http_proxy_port, ftp_proxy_port, disk_cache_size, mem_cache_size;
	gchar *no_proxies_for;
	gchar *http_version = NULL;
	gboolean proxy_keepalive;
	
	gchar *serif_font, *sansserif_font, *monospace_font;
	gchar **font;
	int size_font;
	
	gboolean java_enabled, javascript_enabled;
	int image_loading;
	gchar *background_color, *text_color, *visited_link_color;
	gchar *unvisited_link_color;
	gboolean use_provided_colors, underline_links;

	/* FIXME: these need tidying up desperately */
	
	http_proxy = gnome_config_get_string("/galeon/Advanced/http_proxy=");
	http_proxy_port = gnome_config_get_int("/galeon/Advanced/http_proxy_port=8080");
	disk_cache_size = gnome_config_get_int("/galeon/Advanced/disk_cache_size=7680");
	mem_cache_size = gnome_config_get_int("/galeon/Advanced/mem_cache_size=1024");
	ftp_proxy = gnome_config_get_string("/galeon/Advanced/ftp_proxy=");
	ftp_proxy_port = gnome_config_get_int("/galeon/Advanced/ftp_proxy_port=8080");
	no_proxies_for = gnome_config_get_string("/galeon/Advanced/no_proxies_for=localhost");
	proxy_keepalive = gnome_config_get_bool("/galeon/Advanced/proxy_keepalive=TRUE");
	serif_font = gnome_config_get_string("/galeon/Appearance/serif_font=-adobe-times-medium-r-normal-*-14-*-*-*-p-*-iso8859-1");
	sansserif_font = gnome_config_get_string("/galeon/Appearance/sansserif_font=-adobe-times-medium-r-normal-*-14-*-*-*-p-*-iso8859-1");
	monospace_font = gnome_config_get_string("/galeon/Appearance/monospace_font=-adobe-times-medium-r-normal-*-14-*-*-*-p-*-iso8859-1");
	size_font = gnome_config_get_int("/galeon/Appearance/size_font=14");
	java_enabled = gnome_config_get_bool("/galeon/Advanced/java_enabled=TRUE");
	javascript_enabled = gnome_config_get_bool("/galeon/Advanced/javascript_enabled=TRUE");
	image_loading = gnome_config_get_int("/galeon/Advanced/image_loading_type=0");	
	background_color = gnome_config_get_string("/galeon/Appearance/background_color=#FFFFFF");
	text_color = gnome_config_get_string("/galeon/Appearance/text_color=#000000");
	visited_link_color = gnome_config_get_string
		("/galeon/Appearance/visited_link_color=#FF0000");
	unvisited_link_color = gnome_config_get_string
		("/galeon/Appearance/unvisited_link_color=#0000FF");
	use_provided_colors = (gnome_config_get_int
			       ("/galeon/Appearance/use_provided_colors=0") == 0) 
		? TRUE
		: FALSE;
	underline_links = gnome_config_get_bool
		("/galeon/Appearance/underline_links=TRUE");

	switch (gnome_config_get_int("/galeon/Advanced/http_version=1"))
	{
		case 0: http_version = g_strdup("1.0");
		case 1: http_version = g_strdup("1.1");
	}
      	
	//Set proxy preferences

	if (strlen (http_proxy) != 0)
	{
		mozilla_preference_set_int ("network.proxy.type", 1);
		mozilla_preference_set ("network.proxy.http", http_proxy);
		mozilla_preference_set_int ("network.proxy.http_port", http_proxy_port);
	}

	if (strlen (ftp_proxy) != 0)
	{
		mozilla_preference_set_int ("network.proxy.type", 1);
		mozilla_preference_set ("network.proxy.ftp", ftp_proxy);
		mozilla_preference_set_int ("network.proxy.ftp_port", ftp_proxy_port);
	}

	if ((strlen (http_proxy) != 0) || (strlen (ftp_proxy) != 0))
	{
	        if (strlen (no_proxies_for) != 0) 
		{
		  mozilla_preference_set ("network.proxy.no_proxies_on", no_proxies_for);
		}
		else
		{
		  // Fix problem with changing to blank entry on the fly in preferences
		  mozilla_preference_set ("network.proxy.no_proxies_on", " ");
		}
	}

	mozilla_preference_set_boolean("network.http.proxy.keep-alive", proxy_keepalive);
	
	if (disk_cache_size > 0)
	{
		gchar *cachedir = init_cache_dir();
		mozilla_preference_set("browser.cache.directory", cachedir);
		mozilla_preference_set_boolean("browser.enable.cache", TRUE);
		mozilla_preference_set_int ("browser.cache.disk_cache_size", disk_cache_size);
		g_free(cachedir);
	}
	else
	{
		mozilla_preference_set_boolean("browser.enable.cache", FALSE);
	}

	mozilla_preference_set_int ("browser.cache.memory_cache_size", mem_cache_size);
	
	mozilla_preference_set("browser.display.background_color", background_color);
	mozilla_preference_set("browser.display.foreground_color", text_color);
	mozilla_preference_set("browser.anchor_color", unvisited_link_color);
	mozilla_preference_set("browser.visited_color", visited_link_color);
	mozilla_preference_set_boolean("browser.underline_anchors", underline_links);
	mozilla_preference_set_boolean("browser.display.use_document_colors",
				       use_provided_colors);
 
	font = g_strsplit(serif_font,"-",-1);
	mozilla_preference_set("font.name.serif.x-western",g_strconcat(font[1],"-",font[2],"-",font[13],
								       "-",font[14],NULL));
	g_strfreev(font);

	font = g_strsplit(sansserif_font,"-",-1);
	mozilla_preference_set("font.name.sans-serif.x-western",g_strconcat(font[1],"-",font[2],"-",font[13],
									    "-",font[14],NULL));
	g_strfreev(font);

	font = g_strsplit(monospace_font,"-",-1);
	mozilla_preference_set("font.name.monospace.x-western",g_strconcat(font[1],"-",font[2],"-",font[13],
									   "-",font[14],NULL));
	mozilla_preference_set_int("font.size.fixed.x-western",atoi(font[7]));
	g_strfreev(font);

	mozilla_preference_set_int("font.size.variable.x-western",size_font);

	// note: this might not always work in the future - Nate
	// Actually, it does not work now...
	mozilla_preference_set_boolean ("nglayout.widget.gfxscrollbars", FALSE);
  
	mozilla_preference_set_boolean ("security.checkloaduri", FALSE); 
	mozilla_preference_set_boolean ("security.enable_java", java_enabled); 
	mozilla_preference_set_boolean ("javascript.enabled", javascript_enabled); 
  
	mozilla_preference_set_int("network.image.imageBehavior", image_loading);

	mozilla_preference_set ("general.useragent.misc", "Galeon");
	mozilla_preference_set_int ("network.accept_cookies", 0);
	mozilla_preference_set_int ("network.cookie.cookieBehavior",0);
	mozilla_preference_set_boolean ("network.cookie.warnAboutCookies",FALSE);
	mozilla_preference_set_int ("network.foreign_cookies",             0);

	mozilla_preference_set ("network.http.version", http_version);
	
	mozilla_prefs_set_autodetect_charset (gnome_config_get_int("/galeon/Advanced/autodetect_charset=0"));
	
	mozilla_prefs_set_default_charset ();

	mozilla_prefs_set_languages ();

	g_free (http_proxy);
	g_free (ftp_proxy);
	g_free (no_proxies_for);
	g_free (http_version);
	g_free (background_color);
	g_free (text_color);
	g_free (visited_link_color);
	g_free (unvisited_link_color);
}

void mozilla_prefs_set_autodetect_charset (int charset)
{
	mozilla_preference_set ("intl.charset.detector", autodetect_charset_prefs[charset]);
}

void mozilla_prefs_set_default_charset ()
{
	char *charset; 
	char *charset_code;

	charset = gnome_config_get_string ("/galeon/Advanced/default_charset=Western (ISO-8859-1)");
	charset_code = get_bracketed_string (charset);
	
	if (charset_code)
	{
		mozilla_preference_set ("intl.charset.default", charset_code);
		g_free(charset_code);
	}

	g_free(charset);
}

void mozilla_prefs_set_languages ()
{
	int size;
	gchar **strings = NULL;
	gchar *languages = NULL;
	int k;

	gnome_config_get_vector ("/galeon/Advanced/lang=English\\ (en)", &size, &strings);

	for (k=0; k<size ; k++)
	{	
		char *lang;

		lang = get_bracketed_string (strings[k]);

		if (!lang)
		{
			lang = strings[k];
		}

		if (k==0)
		{
			languages = lang;
		}
		else
		{
			languages = g_strconcat (languages, g_strdup(","), lang, NULL);
		}

	}	

	if (strings)
	{
		mozilla_preference_set ("intl.accept_languages", languages);
		g_free (languages);
	}
}

char * 
get_bracketed_string (char *str)
{
	char *begin;
	char *end;

	begin = strchr (str,'(');
	end = strchr (str,')');

	if (begin && end)
	{
		begin ++;
		return g_strndup(begin, end - begin);
	}
	else 
	{
		return NULL;
	}
}
