/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"

/* local function prototypes */
static void render_from_item(GaleonBrowser *gb, BookMarkItem *b);
static void render_from_list(GaleonBrowser *gb, GList *l);

/*
 * FIXME: this is eventually due to work by writing the HTML into a file,
 * which gets recreated when the bookmarks changed. This functionality
 * can then (in theory) be replaced by an arbitrary external program.
 * -- MattA
 */

/* hackish shorthand to render HTML into the browser */
#define RENDER(str) \
        gtk_moz_embed_append_data(gb->mozEmbed, str, strlen(str))

/**
 * portal_render_into_browser: render the user's bookmarks into a
 * web browser
 */
void
portal_render_into_browser(GaleonBrowser *gb)
{
	gchar *stylesheet_filename, *default_file;
	gchar *configname, full_path[PATH_MAX];
	
	/* check bookmarks are loaded */
	g_return_if_fail(bookmarks_root != NULL);
	
	/* select a suitable style sheet for a default */
	default_file = user_file("myportal.css", FALSE);
	
	/* construct the config name and default */
	configname = g_strconcat("/galeon/Appearance/myportal_stylesheet=",
				 default_file, NULL);
	
	/* get the location of the CSS file for My Portal */
	stylesheet_filename = gnome_config_get_string(configname);

	/* open the rendering stream */
	gtk_moz_embed_open_stream(gb->mozEmbed, MYPORTAL_URL, "text/html");

	/* render the complete portal */
	RENDER("<html><head>\n");
	RENDER("<link rel=\"stylesheet\" href=\"file:");
	RENDER(realpath(stylesheet_filename, full_path) ?
	       full_path : realpath(default_file, full_path));
	RENDER("\" type=\"text/css\">\n");
	RENDER("<title>My Portal</title></head>\n");
	RENDER("<body>\n");
	RENDER("<img class=\"logo\" src=\"file:" SHARE_DIR
	       "/logo.png\" alt=\"\">");
	render_from_item(gb, bookmarks_root);
	RENDER("</body></html>\n");

	/* finish the rendering */
	gtk_moz_embed_close_stream(gb->mozEmbed);
}

/**
 * render_from_item: render a single bookmark item
 */
static void
render_from_item(GaleonBrowser *gb, BookMarkItem *b)
{
	/* could be the case if there are no bookmarks */
	if (b == NULL)
		return;

	/* otherwise do the appropriate thing */
	switch (b->type)
	{
	case CATEGORY:
		RENDER("<blockquote>\n");
		RENDER("<strong>");
		RENDER(b->name);
		RENDER("</strong>\n");
		render_from_list(gb, b->list);
		RENDER("</blockquote>\n");
		break;
		
	case SITE:
		RENDER("<a href=\"");

		/* If url has no preceding "http://" or "ftp://" presume "http://" */
		if (!strstr(b->url, "://")) {
			RENDER("http://");
		}

		RENDER(b->url);
		RENDER("\">");
		RENDER(b->name);
		RENDER("</a>\n");
		break;
		
	case SEPARATOR:
		RENDER("<br>\n");
		break;
	}
}

/**
 * render_from_list: render a list of bookmark items
 */
static void
render_from_list(GaleonBrowser *gb, GList *l)
{
	for (; l != NULL; l = l->next)
	{
		render_from_item(gb, l->data);
	}
}

#undef RENDER

