/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <galeon.h>

void on_menu_ctree_tree_select_row (GtkWidget       *ctree,
				    GtkCTreeNode	*node,
				    GaleonBrowser *browser);
void on_menu_ctree_tree_unselect_row (GtkWidget       *ctree,
				      GList		*node,
				      gint             column,
				      GaleonBrowser   *browser);
void on_dirselect_cancel_clicked (GtkButton *button, GaleonBrowser *browser);
void on_dirselect_ok_clicked (GtkButton *button, GaleonBrowser *browser);
void on_selectdir1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_preferences1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser);
void on_prefs_ok_button_clicked (GtkButton *button, GaleonBrowser *browser);
void on_prefs_apply_button_clicked (GtkButton *button, GaleonBrowser *browser);
void on_prefs_cancel_button_clicked (GtkButton *button, GaleonBrowser *browser);
void on_prefs_add_button_clicked (GtkButton *button, GaleonBrowser *browser);
void on_prefs_delete_button_clicked (GtkButton *button, GaleonBrowser *browser);
void on_prefs_edit_button_clicked (GtkButton *button, GaleonBrowser *browser);
void on_prefs_mime_list_select_row (GtkCList *clist, 
				    gint row, 
				    gint column, 
				    GdkEventButton *event, 
				    GaleonBrowser *browser);
void on_prefs_mime_list_unselect_row (GtkCList *clist,
				      gint row,
				      gint column,
				      GdkEventButton *event,
				      GaleonBrowser *browser);
void on_dirselect2_ok_button_clicked (GtkButton *button, GaleonBrowser *browser);
void on_dirselect2_cancel_button_clicked (GtkButton *button, GaleonBrowser *browser);
void on_homepage_my_portal_button_clicked (GtkButton *button,
					   GaleonBrowser *browser);
void on_homepage_current_button_clicked (GtkButton *button,
					 GaleonBrowser *browser);
void on_lang_add_button_clicked (GtkButton *button, GaleonBrowser *browser);
void on_lang_remove_button_clicked (GtkButton *button, GaleonBrowser *browser);
void on_lang_up_button_clicked (GtkButton *button, GaleonBrowser *browser);
void on_lang_down_button_clicked (GtkButton *button, GaleonBrowser *browser);
void on_clear_cache_button_clicked (GtkButton *button, GaleonBrowser *browser);

MimeItem *selected_mime;

/**
 * on_dirselect_cancel_clicked: download directory dialog cancel clicked (menu)
 */
void
on_dirselect_cancel_clicked (GtkButton *button, GaleonBrowser *browser)
{
	GtkWidget *dirselect = glade_lookup_widget(GTK_WIDGET(button), "dirselection");
	gtk_widget_destroy(dirselect);
}

/**
 * on_dirselect_ok_clicked: download directory dialog ok clicked
 */
void
on_dirselect_ok_clicked (GtkButton *button, GaleonBrowser *browser)
{
	GtkWidget *dirselect = glade_lookup_widget(GTK_WIDGET(button), "dirselection");
	char * download_dir = gtk_file_selection_get_filename(GTK_FILE_SELECTION(dirselect));
	
	gnome_config_set_string("/galeon/Downloading/download_dir", download_dir);
	gnome_config_sync();
  
	gtk_widget_destroy(GTK_WIDGET(dirselect));
}

/**
 * on_selectdir1_activate: show the download directory dialog (menu)
 */
void
on_selectdir1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
	GtkWidget *dirselect;
	char *olddir;
	GladeXML *gxml;

	gxml = glade_xml_new(glade_file(), "dirselection");
	glade_xml_signal_autoconnect_full (gxml, (GladeXMLConnectFunc) glade_signal_connect_func, browser);
	dirselect = glade_xml_get_widget(gxml, "dirselection");

	olddir = gnome_config_get_string("/galeon/Downloading/download_dir");

	gtk_file_selection_set_filename(GTK_FILE_SELECTION(dirselect), olddir);
  
	gtk_widget_set_sensitive(GTK_WIDGET(GTK_FILE_SELECTION(dirselect)->file_list), FALSE);
	gtk_widget_show(dirselect);
}

/**
 * on_preferences1_activate: show preferences dialog
 */
void
on_preferences1_activate (GtkMenuItem *menuitem, GaleonBrowser *browser)
{
  GladeXML *gxml;
  gxml = glade_xml_new(glade_file(), "Preferences");
  glade_xml_signal_autoconnect_full (gxml, (GladeXMLConnectFunc) glade_signal_connect_func, browser);
  prefs = glade_xml_get_widget(gxml, "Preferences");
  
  preferences_fill(prefs);
  gtk_widget_show(prefs);
}

/**
 * on_prefs_ok_button_clicked: preferences dialog ok button clicked
 */
void
on_prefs_ok_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
  GtkWidget *prefs = glade_lookup_widget(GTK_WIDGET(button), "Preferences");
  preferences_process(prefs);
  mozilla_prefs_set ();
  set_toolbar_style_global();
  gtk_widget_destroy(prefs);
}

/**
 * on_prefs_ok_button_clicked: preferences dialog ok button clicked
 */
void
on_clear_cache_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
	gchar *confdir = g_strconcat(g_get_home_dir(),"/.galeon",NULL);
	gchar *cachedir = g_strconcat(confdir,"/cache",NULL);

	remove_directory (cachedir, TRUE);

	g_free (confdir);
	g_free (cachedir);
}


/**
 * on_prefs_apply_button_clicked: preferences dialog apply button clicked
 */
void
on_prefs_apply_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
  GtkWidget *prefs = glade_lookup_widget (GTK_WIDGET(button), "Preferences");
  preferences_process (prefs);
  set_toolbar_style_global ();  
  mozilla_prefs_set ();
}


void
on_prefs_cancel_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
  // This is ok for now, but we also need a 'revert' feature
  GtkWidget *prefs = glade_lookup_widget(GTK_WIDGET(button), "Preferences");
  gtk_widget_destroy(prefs);
}

/**
 * on_prefs_add_button_clicked: mime types add button clicked
 */
void
on_prefs_add_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
	gchar *name, *program, *extensions;
	MimeAction action;

	mime_get_entry(&name, &program, &extensions, &action);

	if (strlen(name) == 0)
		return;
  
	mime_add(name, program, extensions, action);
	preferences_mime_update();
}


/**
 * on_prefs_delete_button_clicked: mime types delete button clicked
 */
void
on_prefs_delete_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
	if (!selected_mime)
		return;
	
	mime_delete(selected_mime);  
	preferences_mime_update();
	
	selected_mime = NULL;
}


/**
 * on_prefs_edit_button_clicked: mime types edit button clicked
 */
void
on_prefs_edit_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
	gchar *name, *program, *extensions;
	MimeAction action;

	if (!selected_mime)
		return;
	
	mime_get_entry(&name, &program, &extensions, &action);

	mime_update(selected_mime, name, program, extensions, action);
	preferences_mime_update();
}

/**
 * on_prefs_mime_list_select_row: mime type selected
 */
void on_prefs_mime_list_select_row (GtkCList *clist, 
				    gint row, 
				    gint column, 
				    GdkEventButton *event, 
				    GaleonBrowser *browser)
{
	GtkWidget *name_entry = glade_lookup_widget(GTK_WIDGET(clist), "name_entry");
	GtkWidget *ext_entry = glade_lookup_widget(GTK_WIDGET(clist), "ext_entry");
	GtkWidget *program_entry = glade_lookup_widget(GTK_WIDGET(clist), "program_entry");
	GtkWidget *action_optionmenu = glade_lookup_widget(GTK_WIDGET(clist), "action_optionmenu");
	gchar *extension;

	selected_mime = (MimeItem*) gtk_clist_get_row_data(clist, row);
  
	extension = g_strjoinv (",", selected_mime->extensions);
	gtk_option_menu_set_history(GTK_OPTION_MENU(action_optionmenu), selected_mime->action);
	gtk_entry_set_text(GTK_ENTRY(name_entry), selected_mime->name);
	gtk_entry_set_text(GTK_ENTRY(ext_entry), extension);
	gtk_entry_set_text(GTK_ENTRY(program_entry), selected_mime->program);

	g_free(extension);
}

/**
 * on_prefs_mime_list_unselect_row: mime type unselected
 */
void on_prefs_mime_list_unselect_row   (GtkCList *clist,
					gint row,
					gint column,
					GdkEventButton *event,
					GaleonBrowser *browser)
{
  GtkWidget *name_entry = glade_lookup_widget(GTK_WIDGET(clist), "name_entry");
  GtkWidget *ext_entry = glade_lookup_widget(GTK_WIDGET(clist), "ext_entry");
  GtkWidget *program_entry = glade_lookup_widget(GTK_WIDGET(clist), "program_entry");
  GtkWidget *action_optionmenu = glade_lookup_widget(GTK_WIDGET(clist), "action_optionmenu");

  selected_mime = NULL;
  
  gtk_option_menu_set_history(GTK_OPTION_MENU(action_optionmenu), 0);
  gtk_entry_set_text(GTK_ENTRY(name_entry), "");
  gtk_entry_set_text(GTK_ENTRY(ext_entry), "");
  gtk_entry_set_text(GTK_ENTRY(program_entry), "");

}

/**
 * on_dirselect2_ok_clicked: download directory dialog ok clicked (prefs dialog)
 */
void
on_dirselect2_ok_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
  GtkWidget *dirselect = glade_lookup_widget(GTK_WIDGET(button), "dirselection2");
  char * download_dir = gtk_file_selection_get_filename(GTK_FILE_SELECTION(dirselect));
  char *url = gtk_object_get_data(GTK_OBJECT(dirselect), "url");

  gnome_config_push_prefix("/galeon/Downloading/");
 
  gnome_config_set_string("download_dir", download_dir);
  gnome_config_pop_prefix();
  gnome_config_sync();

  save_url_with_gtm (url);
  
  g_free(url);
  
  gtk_widget_destroy(GTK_WIDGET(dirselect));
}

/**
 * on_dirselect2_cancel_clicked: download directory dialog cancel clicked (prefs dialog)
 */
void
on_dirselect2_cancel_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
  GtkWidget *dirselect = glade_lookup_widget(GTK_WIDGET(button), "dirselection2");
  gtk_widget_destroy(dirselect);
}


/**
 * on_menu_ctree_tree_select_row: select a prefs category
 */
void
on_menu_ctree_tree_select_row          (GtkWidget       *ctree,
                                        GtkCTreeNode	*node,
					GaleonBrowser *browser)
{
  GtkWidget *notebook = glade_lookup_widget(ctree, "notebook1");
  int row = 0;

  row = (gint)gtk_ctree_node_get_row_data(GTK_CTREE(ctree), GTK_CTREE_NODE(node));
  gtk_notebook_set_page(GTK_NOTEBOOK(notebook), row);
}


/**
 * on_menu_ctree_tree_unselect_row: unselect a prefs category
 */
void
on_menu_ctree_tree_unselect_row        (GtkWidget       *ctree,
                                        GList		*node,
					gint             column,
                                        GaleonBrowser   *browser)
{
  GtkWidget *notebook = glade_lookup_widget(ctree, "notebook1");
  gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 0);
}


/**
 * on_homepage_my_portal_button_clicked: set homepage URL to "My Portal"
 */
void
on_homepage_my_portal_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
  GtkWidget *entry;

  entry = glade_lookup_widget(GTK_WIDGET(button), "startpage_entry");
  gtk_entry_set_text(GTK_ENTRY(entry), MYPORTAL_URL);
}


/**
 * on_homepage_current_button_clicked: set homepage URL to current page
 */
void
on_homepage_current_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
  GtkWidget *entry;
  gchar *url;

  url = history_get_last_url();
  entry = glade_lookup_widget(GTK_WIDGET(button), "startpage_entry");
  gtk_entry_set_text(GTK_ENTRY(entry), url);
  g_free(url);
}

void
on_lang_add_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
	GtkList *list = GTK_LIST(glade_lookup_widget (GTK_WIDGET(button),"lang_list"));
	GtkWidget *entry = glade_lookup_widget (GTK_WIDGET(button),"lang_entry");
	GList *items = NULL;
	GtkWidget *list_item;
	char *entry_text;

	entry_text = gtk_editable_get_chars(GTK_EDITABLE(entry),0,-1);
	
	list_item = gtk_list_item_new_with_label(entry_text);
	gtk_object_set_data(GTK_OBJECT(list_item), "string", entry_text);

	items = g_list_append (items, list_item);
	gtk_list_append_items(list, items);
	gtk_widget_show(list_item);
}

void
on_lang_remove_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
	GtkList *list = GTK_LIST(glade_lookup_widget (GTK_WIDGET(button),"lang_list"));
	GList *selected;

	if (list->selection)
	{
		selected = g_list_copy(list->selection);
		gtk_list_remove_items (list, selected); 
	}
}

void
on_lang_up_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
	GtkList *list = GTK_LIST(glade_lookup_widget (GTK_WIDGET(button),"lang_list"));
	GList *selected;
	int pos;

	if (list->selection)
	{
		selected = g_list_copy(list->selection);
		pos = gtk_list_child_position(list, GTK_WIDGET(selected->data));
		if (pos!=0)
		{
			gtk_list_remove_items_no_unref (list, selected); 
			pos --;
			gtk_list_insert_items (list,selected,pos);
			gtk_list_select_item (list,pos);
		}
	}
}

void
on_lang_down_button_clicked (GtkButton *button, GaleonBrowser *browser)
{
	GtkList *list = GTK_LIST(glade_lookup_widget (GTK_WIDGET(button),"lang_list"));
	GList *selected;
	int pos;

	if (list->selection)
	{
		selected = g_list_copy(list->selection);
		pos = gtk_list_child_position(list, GTK_WIDGET(selected->data));
		gtk_list_remove_items_no_unref (list, selected); 
		pos ++;
		gtk_list_insert_items (list, selected, pos);
		gtk_list_select_item (list, pos);
	}
	
}

